package physbeans.editors;

import java.awt.*;
import javax.swing.*;
import java.beans.*;
import physbeans.inout.*;
import physbeans.math.*;
import static java.lang.Math.*;

/**
 * The panel for the IVectorEditor.
 * <p>It shows a list of the array values and allows to change them and the size
 * of the vector.
 */
public class IVectorCustomEditor extends JPanel {
  
  protected PropertyEditorSupport editor;
  protected NumericTextField[] textfield;
  protected NumericTextField dimField;
  protected Box pList;
  protected Font standardFont;
  
  protected final int cols = 12;   // size of text fields
  protected final int prec = 8;   // precision of text fields
  
  public IVectorCustomEditor(PropertyEditorSupport ed) {
    // get the old values
    editor = ed;
    IVector value = (IVector) editor.getValue();
    int dim = (value == null) ? 0 : value.getDimension();
    
    // vertical list with values, on top box for dimension
    standardFont = new Font("Lucida Sans", Font.PLAIN, 11);
    setLayout(new BorderLayout());
    JPanel pHead = new JPanel();
    dimField = new NumericTextField(dim, 8, 4, "Dimension = ", "");
    dimField.setFont(standardFont);
    pHead.add(dimField);
    dimField.addPropertyChangeListener(new MyDimListener());
    
    // create panel with text fields
    pList = new Box(BoxLayout.Y_AXIS);
    pList.setPreferredSize(new Dimension(260, dim*30+30));
    JScrollPane scrollPane = new JScrollPane(pList,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    textfield = new NumericTextField[0];
    setTextFieldsToVector(value);
    
    add(pHead, BorderLayout.NORTH);
    add(scrollPane, BorderLayout.CENTER);
    setMaximumSize(new Dimension(280, 150));
  }
  
  /**
   * Called when the value of a textfield is changed.
   */
  protected void changeValue() {
    IVector newValue = getValueFromTextFields();
    
    editor.setValue(newValue);
    editor.firePropertyChange();
  }
  
  /**
   * Called when the value of the dimension field is changed.
   */
  protected void resize() {
    int newDim = (int) Math.round(dimField.getValue());
    
    if (newDim < 0) {
      newDim = textfield.length;
    }
    
    if (newDim != textfield.length) {
      IVector newValue = new IVector(newDim);
      setTextFieldsToVector(newValue);
      editor.setValue(newValue);
      editor.firePropertyChange();
    }
    
    dimField.setValueInternally(newDim);   // to fix fractional input
  }
  
  /**
   * Gets IVector from the text fields.
   */
  protected IVector getValueFromTextFields() {
    int dim = textfield.length;
    IVector newValues = new IVector(dim);
    
    for (int i = 0; i < dim; i++) {
      newValues.set(i, (int) round(textfield[i].getValue()));
    }
    
    return newValues;
  }
  
  /**
   * Sets text fields from IVector.
   */
  protected void setTextFieldsToVector(IVector newVal) {
    int dim = (newVal == null) ? 0 : newVal.getDimension();
    
    if (textfield.length == dim) {
      for (int i = 0; i < dim; i++) {
        textfield[i].setValueInternally(newVal.get(i));
      }
    } else {
      pList.removeAll();
      textfield = new NumericTextField[dim];
      for (int i = 0; i < dim; i++) {
        textfield[i] = new NumericTextField(newVal.get(i), cols, prec,
            "x(" + i + ") = ", "");
        textfield[i].setFont(standardFont);
        pList.add(textfield[i]);
        textfield[i].addPropertyChangeListener(new MyTextListener());
      }
      pList.setPreferredSize(new Dimension(260, dim*30+30));
    }
    validate();
  }
  
  /**
   * Listeners to change in the text fields.
   */
  public class MyTextListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      changeValue();
    }
  }
  
  /**
   * Listeners to change in the dimension field.
   */
  public class MyDimListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      resize();
    }
  }
  
}
