package physbeans.editors;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import physbeans.inout.*;
import physbeans.math.*;

/**
 * The panel for the DVectorEditor.
 * <p>It shows a list of the array values and allows to change them and the size
 * of the vector.
 */
public class PolynomialCustomEditor extends JPanel {
  
  protected PropertyEditorSupport editor;
  protected NumericTextField[] textfield;
  protected NumericTextField degField;
  protected JLabel polyField;
  protected Box pList;
  protected Font standardFont;
  
  protected final int cols = 12;   // size of text fields
  protected final int prec = 8;   // precision of text fields
  
  public PolynomialCustomEditor(PropertyEditorSupport ed) {
    // get the old values
    editor = ed;
    Polynomial value = (Polynomial) editor.getValue();
    int dim = value.getDegree() + 1;
    
    // vertical list with values, on top box for dimension, on bottom
    // a string representation of the polynomial
    standardFont = new Font("Lucida Sans", Font.PLAIN, 11);
    setLayout(new BorderLayout());
    JPanel pHead = new JPanel();
    degField = new NumericTextField(dim-1, 8, 4, "Degree = ", "");
    degField.setFont(standardFont);
    pHead.add(degField);
    degField.addPropertyChangeListener(new MyDimListener());
    
    // create panel with text fields
    pList = new Box(BoxLayout.Y_AXIS);
    pList.setPreferredSize(new Dimension(260, dim*30));
    JScrollPane scrollPane = new JScrollPane(pList,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    textfield = new NumericTextField[0];
    polyField = new JLabel();
    polyField.setFont(standardFont);
    setTextFieldsToValue(value);
    
    add(pHead, BorderLayout.NORTH);
    add(scrollPane, BorderLayout.CENTER);
    add(polyField, BorderLayout.SOUTH);
    setMaximumSize(new Dimension(280, 150));
  }
  
  /**
   * Called when the value of a textfield is changed.
   */
  protected void changeValue() {
    Polynomial newVal = getValueFromTextFields();
    polyField.setText(newVal.toString());    
    
    editor.setValue(newVal);
    editor.firePropertyChange();
  }
  
  /**
   * Called when the value of the degree field is changed.
   */
  protected void resize() {
    int newDim = (int) Math.round(degField.getValue()) + 1;
    
    if (newDim < 0) {
      newDim = textfield.length;
    }
    
    if (newDim != textfield.length) {
      Polynomial newValue = new Polynomial(newDim - 1);
      setTextFieldsToValue(newValue);
      editor.setValue(newValue);
      editor.firePropertyChange();
    }
    
    degField.setValueInternally(newDim - 1);   // to fix wrong input
  }
  
  /**
   * Gets polynomial from the text fields.
   */
  protected Polynomial getValueFromTextFields() {
    int dim = textfield.length;
    DVector newValues = new DVector(dim);
    
    for (int i = 0; i < dim; i++) {
      newValues.set(i, textfield[i].getValue());
    }
    
    return new Polynomial(newValues);
  }
  
  /**
   * Sets text fields from polynomial.
   */
  protected void setTextFieldsToValue(Polynomial newVal) {
    DVector coeff = newVal.getCoeff();
    int dim = coeff.getDimension();
    
    if (textfield.length == dim) {
      for (int i = 0; i < dim; i++) {
        textfield[i].setValueInternally(coeff.get(i));
      }
    } else {
      pList.removeAll();
      textfield = new NumericTextField[dim];
      for (int i = 0; i < dim; i++) {
        textfield[i] = new NumericTextField(coeff.get(i), cols, prec,
            "a(" + (dim - i - 1) + ") = ", "");
        textfield[i].setFont(standardFont);
        pList.add(textfield[i]);
        textfield[i].addPropertyChangeListener(new MyTextListener());
      }
      pList.setPreferredSize(new Dimension(260, dim*30));
    }
    
    polyField.setText(newVal.toString());
    validate();
  }
  
  /**
   * Listeners to change in the text fields.
   */
  public class MyTextListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      changeValue();
    }
  }
  
  /**
   * Listeners to change in the dimension field.
   */
  public class MyDimListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      resize();
    }
  }
  
}
