package physbeans.func;

import java.beans.*;
import physbeans.math.*;

/**
 * ParsedFunction that checks the input variables according to a boolean
 * condition.
 *
 * <p>The condition is defined by a string that is parsed in the usual way.
 * If the evaluation of the condition using the input values gives false, then
 * the call is rejected and the function sends a veto event back to the sender.
 */
public class CheckedFunction extends ParsedFunction {
  
  protected String checkString;
  protected SuryonoParser checkParser;
  
  /**
   * creates a standard CheckedFunction
   */
  public CheckedFunction() {
    super();
    checkString = "true";
    createCheckParser();
  }
  
  public void checkValue(int index, PropertyChangeEvent event)
  throws PropertyVetoException {
    // problem: model of input is double[], but often it is assembled from
    //          scalars coming from NTFs
    
    // create copy of input values with the one new value changed
    DVector newVals = inputValues.copy();
    newVals.set(index, ((Double) event.getNewValue()).doubleValue());    
     
    if (! inputOK(newVals)) {
      throw new PropertyVetoException("CheckedFunction", event);
    }
  }
  
  /**
   * Gets the string defining the check function.
   */
  public String getCheckString() {
    return checkString;
  }
  
  /**
   * Sets the string defining the check function.
   */
  public void setCheckString(String s) {
    checkString = s ;
    createCheckParser();
  }
  
  /**
   * Sets the names of all variables.
   */
  public void setVariableName(String[] names) {
    super.setVariableName(names);
    createCheckParser();
  }
  
  /**
   * Creates parser for the check function.
   */
  protected void createCheckParser() {
    int nVars = vars.length;
    
    if (checkParser == null || nVars != checkParser.getVariableNames().length) {
      checkParser = new SuryonoParser(nVars);
    }
    
    for (int v = 0; v < nVars; v++) {
      checkParser.defineVariable(v+1,vars[v]);
    }
    
    String s = "if(" + checkString + ", 1.0, 0.0)";
    checkParser.define(s);
    checkParser.parse();
  }
  
  /**
   * Checks whether the new input values are ok.
   **/
  protected boolean inputOK(DVector in) {
    // since the parser only returns doubles, the result has to be mapped to
    // boolean in the standard way
    
    // set variables from input vector
    int nVars = vars.length;
    int nPars = parameter.getDimension();
    
    for (int v = 0; v < nVars - nPars; v++) {
      checkParser.setVariable(v+1, in.get(v));
    }
    
    // set parameters
    for (int p = 0; p < nPars; p++) {
      checkParser.setVariable(nVars - nPars + p + 1, parameter.get(p));
    }
    
    return checkParser.evaluate() == 1.0 ? true : false;
  }
}
