package physbeans.func;

import physbeans.model.*;
import physbeans.math.*;
import physbeans.inout.*;
import static java.lang.Math.*;
import static physbeans.math.Maths.sqr;

/**
 * Function that makes a Real2DFunction from a Complex2DFunction.
 * The conversion works by returning the real, imaginary, absolute value, 
 * square of the absolute value or the phase of the complex result.
 * If a screen is defined, it's limits are automatically adjusted according
 * to the conversion type.
 */
public class ComplexToRealFunctionAdapter extends GenericFunction
        implements Real2DFunction {

  protected Complex2DFunction func;
  protected ComplexToRealEnum type;
  protected ControlledLayeredScreen screen;
  protected double yMin, yMax;
  protected double cutoff;

  public ComplexToRealFunctionAdapter() {
    super();
    type = ComplexToRealEnum.REAL;
    yMin = -1.0;
    yMax = 1.0;
    cutoff = 0.0;
  }

  @Override
  protected void compute() {
  }

  /**
   * Returns scalar field value according to its conversion type.
   */
  public double getScalarValue(double x, double y) {
    if (func == null) {
      return Double.NaN;
    }

    Complex z = func.getComplexValue(x, y);

    switch (type) {
      case REAL:
        return z.real();
      case IMAG:
        return z.imag();
      case ABS:
        return hypot(z.real(), z.imag());
      case ABS2:
        return sqr(z.real()) + sqr(z.imag());
      case PHASE:
        if (hypot(z.real(), z.imag()) > cutoff) {
          return atan2(z.imag(), z.real());
        } else {
          return 0.0;
        }
      default:
        return z.real();
    }
  }

  /**
   * Gets the complex function that is adapted to a real function.
   */
  public Complex2DFunction getComplexFunction() {
    return func;
  }

  /**
   * Sets the complex function that is adapted to a real function.
   */
  public void setComplexFunction(Complex2DFunction f) {
    func = f;
    if (autoTriggered) {
      trigger();
    }
  }

  /**
   * Gets the conversion type of the adapter.
   * Possible values are
   * <ul>
   * <li>REAL: return real values</li>
   * <li>IMAG: return imaginary values</li>
   * <li>ABS: return absolute values</li>
   * <li>PHASE: return phase values</li>
   * </ul>
   * Default is ABS.
   */
  public ComplexToRealEnum getConversionType() {
    return type;
  }

  /**
   * Sets the conversion type of the adapter.
   * Adjust the sizes of a screen accordingly.
   */
  public void setConversionType(ComplexToRealEnum t) {
    type = t;
    adjustScreen();
    if (autoTriggered) {
      trigger();
    }
  }

  /**
   * Sets the conversion type of the adapter using its ordinal number.
   */
  public void setConversionTypeFromInt(int n) {
    setConversionType(ComplexToRealEnum.values()[n]);
  }

  /**
   * Gets the screen or similar object that is automatically adjusted by a
   * change of the conversion type.
   */
  public ControlledLayeredScreen getScreen() {
    return screen;
  }

  /**
   * Sets the screen or similar object that is automatically adjusted by a
   * change of the conversion type.
   */
  public void setScreen(ControlledLayeredScreen s) {
    screen = s;
    adjustScreen();
  }

  /**
   * Sets the maximal y coordinate of the screen for real/imaginary display.
   */
  public double getYMax() {
    return yMax;
  }

  /**
   * Sets the maximal y coordinate of the screen for real/imaginary display.
   */
  public void setYMax(double y) {
    yMax = y;
    adjustScreen();
  }

  /**
   * Sets the minimal y coordinate of the screen for real/imaginary display.
   */
  public double getYMin() {
    return yMin;
  }

  /**
   * Sets the minimal y coordinate of the screen for real/imaginary display.
   */
  public void setYMin(double y) {
    yMin = y;
    adjustScreen();
  }

  /**
   * Sets cutoff for phase computation.
   * If the absolute value is small, the phase values fluctuate wildly. To get
   * better plots, phase values are set to 0, where the absolute value is
   * smaller than the cutoff.
   */
  public void setCutoff(double c) {
    cutoff = c;
  }

  /**
   * Gets cutoff for phase computation.
   */
  public double getCutoff() {
    return cutoff;
  }

  protected void adjustScreen() {
    if (screen != null) {
      screen.setInitialScaleIndex(4);
      switch (type) {
        case REAL:
        case IMAG:
          screen.setMinY(yMin);
          screen.setMaxY(yMax);
          break;
        case ABS:
          screen.setMinY(0);
          screen.setMaxY(hypot(yMin, yMax));
          break;
        case ABS2:
          screen.setMinY(0);
          screen.setMaxY(sqr(yMin) + sqr(yMax));
          break;
        case PHASE:
          screen.setMinY(-3.16);  // slightly more than PI to cope with pixel size
          screen.setMaxY(3.16);
          break;
        default:
          screen.setMinY(yMin);
          screen.setMaxY(yMax);
      }
    }
  }
}
