package physbeans.func;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Function that computes a number of field values of a given scalar field
 *   along a horizontal or vertical cut line<br>
 * model: ScalarField   (field to cut through)
 * vector input:  none
 * vector output: x, y (set of points, each one of the form (x, sf(x))
 */
public class CutFunction extends GenericVectorFunction {
  
  protected boolean vertical;
  protected double y, xMin, xMax;
  protected Real2DFunction field;
  
  /** Creates a new instance of CutFunction */
  public CutFunction() {
    super(0, 2, 100);
    xMin = -1.0;
    xMax = 1.0;
    y = 0.0;
    vertical = false;
    field = null;
  }
  
  /**
   * Computes new values for the output vectors.
   */
  protected void compute() {
    if (field == null) {
      return;
    }
    
    int n = getNPoints();
    double dx = (xMax - xMin)/(n - 1);
    
    double x = xMin;
    for(int i = 0; i < n; i++) {
      outputValues[0].set(i, x);
      if (vertical) {
        outputValues[1].set(i, field.getScalarValue(y, x));
      } else {
        outputValues[1].set(i, field.getScalarValue(x, y));
      }
      x += dx;
    }
  }
  
  /**
   * Gets value of vertical flag.<br>
   * If true, cut is vertical instead of horizontal.
   */
  public boolean isVertical() {
    return this.vertical;
  }
  
  /**
   * Sets value of vertical flag.<br>
   * If true, cut is vertical instead of horizontal.
   */
  public void setVertical(boolean vertical) {
    this.vertical = vertical;
    trigger();
  }
  
  /**
   * Gets constant coordinate defining cut.<br>
   * For vertical cuts it is actually an x value.
   */
  public double getY() {
    return this.y;
  }
  
  /**
   * Sets constant coordinate defining cut.<br>
   * For vertical cuts it is actually an x value.
   */
  public void setY(double y) {
    this.y = y;
    trigger();
  }
  
  /**
   * Gets start value of cut. <br>
   * For vertical cuts it is actually an y value
   */
  public double getXMin() {
    return this.xMin;
  }
  
  /**
   * Sets start value of cut. <br>
   * For vertical cuts it is actually an y value
   */
  public void setXMin(double xMin) {
    this.xMin = xMin;
    trigger();
  }
  
  /**
   * Gets end value of cut. <br>
   * For vertical cuts it is actually an y value
   */
  public double getXMax() {
    return this.xMax;
  }
  
  /**
   * Sets end value of cut. <br>
   * For vertical cuts it is actually an y value
   */
  public void setXMax(double xMax) {
    this.xMax = xMax;
    trigger();
  }
  
  /**
   * Gets the number of points computed along the cut.
   */
  public int getNPoints() {
    return getOutputLength(0);
  }
  
  /**
   * Sets the number of points computed along the cut.
   */
  public void setNPoints(int nPoints) {
    setAllOutputLengths(nPoints);
  }
  
  /**
   * Returns the values of the independent coordinate (x for a horizontal cut)
   * as a vector.
   */
  public DVector getXVector() {
    return outputValues[0];
  }
  
  /**
   * Returns the values of the dependent coordinate (the scalar field along the
   * cut) as a vector.
   */
  public DVector getYVector() {
    return outputValues[1];
  }
  
  /**
   * Gets the scalar field the CutFunction cuts through.
   */
  public Real2DFunction getScalarField() {
    return field;
  }
  
  /**
   * Sets the scalar field the CutFunction cuts through.
   */
  public void setScalarField(Real2DFunction f) {
    field = f;
    trigger();
  }
}
