package physbeans.inout;

import physbeans.model.*;
import physbeans.event.*;
import physbeans.views.*;
import physbeans.math.*;
import java.awt.*;

/**
 * LayeredScreen with additional ScreenControls<br>
 * Conveniance bean which packages the two coupled beans that are often used
 * together.
 */
public class ControlledLayeredScreen extends PhysicsPanel
        implements SizeAdjustable {

  protected boolean scaleX;
  protected boolean scaleY;
  protected LayeredScreen screen;
  protected ScreenControls controls;

  /**
   * Creates a new instance of ControlledLayeredScreen
   */
  public ControlledLayeredScreen() {
    super();
    scaleX = false;
    scaleY = true;

    screen = new LayeredScreen();
    controls = new ScreenControls();

    setCrossCursor(true);
    setScreenOpaque(true);
    setScreenBackground(Color.WHITE);

    setLayout(new BorderLayout());
    add(screen, BorderLayout.CENTER);
    add(controls, BorderLayout.SOUTH);

    screen.addVectorListener(new VectorListener() {

      public void valueChanged(VectorEvent evt) {
        controls.setTextFields(screen.getClickedPosition());
      }
    });

    controls.addVectorListener(new VectorListener() {

      public void valueChanged(VectorEvent evt) {
        if (scaleX) {
          screen.changeXScale(controls.getScaleChange());
        }
        if (scaleY) {
          screen.changeYScale(controls.getScaleChange());
        }
      }
    });
  }

  // route through all necessary properties
  /**
   * Sets parameters when the layout (basically the size) is fixed.
   */
  public void doLayout() {
    super.doLayout();
    screen.doLayout();
  }

  /**
   * Sets label of scale controls.
   */
  public void setScaleLabel(String s) {
    controls.setScaleLabel(s);
  }

  /**
   * Gets label of scale controls.
   */
  public String getScaleLabel() {
    return controls.getScaleLabel();
  }

  /**
   * Sets label of second scale controls.
   */
  public void setScaleLabel2(String s) {
    controls.setScaleLabel2(s);
  }

  /**
   * Gets label of second scale controls.
   */
  public String getScaleLabel2() {
    return controls.getScaleLabel2();
  }

  /**
   * Sets label of position text fields.
   */
  public void setPositionLabel(String s) {
    controls.setPositionLabel(s);
  }

  /**
   * Gets label of position text fields.
   */
  public String getPositionLabel() {
    return controls.getPositionLabel();
  }

  /**
   * Sets initial value of the scale index.
   */
  public void setInitialScaleIndex(int newIndex) {
    controls.setInitialScaleIndex(newIndex);
  }

  /**
   * Gets initial value of the scale index.
   */
  public int getInitialScaleIndex() {
    return controls.getInitialScaleIndex();
  }

  /**
   * Sets initial value of the second scale index.
   */
  public void setInitialScaleIndex2(int newIndex) {
    controls.setInitialScaleIndex2(newIndex);
  }

  /**
   * Gets initial value of the second scale index.
   */
  public int getInitialScaleIndex2() {
    return controls.getInitialScaleIndex2();
  }

  /**
   * Gets cursor flag: true for crosshair, false for default cursor.
   */
  public boolean isCrossCursor() {
    return screen.isCrossCursor();
  }

  /**
   * Sets cursor flag: true for crosshair, false for default cursor.
   */
  public void setCrossCursor(boolean crossCursor) {
    screen.setCrossCursor(crossCursor);
  }

  /**
   * Gets maximal world x coordinate.
   */
  public double getMaxX() {
    return screen.getMaxX();
  }

  /**
   * Sets maximal world x coordinate.
   */
  public void setMaxX(double xMax) {
    screen.setMaxX(xMax);
  }

  /**
   * Gets minimal world x coordinate.
   */
  public double getMinX() {
    return screen.getMinX();
  }

  /**
   * Sets minimal world x coordinate.
   */
  public void setMinX(double xMin) {
    screen.setMinX(xMin);
  }

  /**
   * Gets maximal world y coordinate.
   */
  public double getMaxY() {
    return screen.getMaxY();
  }

  /**
   * Sets maximal world y coordinate.
   */
  public void setMaxY(double yMax) {
    screen.setMaxY(yMax);
  }

  /**
   * Gets minimal world y coordinate.
   */
  public double getMinY() {
    return screen.getMinY();
  }

  /**
   * Sets minimal world y coordinate.
   */
  public void setMinY(double yMin) {
    screen.setMinY(yMin);
  }

  /**
   * Returns true, if x and y scale are equal.
   */
  public boolean isEqualScales() {
    return screen.isEqualScales();
  }

  /**
   * Forces x and y scales to be equal or releases this constraint.
   */
  public void setEqualScales(boolean equalScales) {
    screen.setEqualScales(equalScales);
  }

  /**
   * Adds a view to the list of views.
   */
  public void addView(View v) {
    screen.addView(v);
  }

  /**
   * Gets status of view with no. index.
   */
  public boolean getViewActive(int index) {
    return screen.getViewActive(index);
  }

  /**
   * Gets status of all views.
   */
  public boolean[] getViewActive() {
    return screen.getViewActive();
  }

  /**
   * Sets status of view of no. index.
   */
  public void setViewActive(int index, boolean viewActive) {
    screen.setViewActive(index, viewActive);
  }

  /**
   * Sets status of all views.
   */
  public void setViewActive(boolean[] viewActive) {
    screen.setViewActive(viewActive);
  }

  /**
   * Sets background color of the screen.<br>
   * The background color of the controls is set by the surrounding panel.
   */
  public void setScreenBackground(Color col) {
    screen.setBackground(col);
  }

  /**
   * Gets background color of the screen.<br>
   * The background color of the controls is set by the surrounding panel.
   */
  public Color getScreenBackground() {
    return screen.getBackground();
  }

  /**
   * Returns true, if the screen is opaque.
   */
  public boolean isScreenOpaque() {
    return screen.isOpaque();
  }

  /**
   * Sets the opacity of the screen.
   */
  public void setScreenOpaque(boolean flag) {
    if (screen != null) {
      screen.setOpaque(flag);
    }
  }

  /**
   * Gets value of scaleX flag.<br>
   * If true, the scaling factor is applied to the x direction
   */
  public boolean isScaleX() {
    return this.scaleX;
  }

  /**
   * Sets value of scaleX flag.<br>
   * If true, the scaling factor is applied to the x direction
   */
  public void setScaleX(boolean scaleX) {
    this.scaleX = scaleX;
  }

  /**
   * Gets value of scaleY flag.<br>
   * If true, the scaling factor is applied to the y direction
   */
  public boolean isScaleY() {
    return this.scaleY;
  }

  /**
   * Sets value of scaleY flag.<br>
   * If true, the scaling factor is applied to the y direction
   */
  public void setScaleY(boolean scaleY) {
    this.scaleY = scaleY;
  }

  /**
   * Simulates a mouse click for testing purposes.
   */
  public void doClick(Point p) {
    screen.doClick(p);
  }

  /**
   * Gets position of last mouse click in world coordinates.
   */
  public DVector getClickedPosition() {
    return screen.getClickedPosition();
  }

  /**
   * Gets flag whether two scale panels are supplied.
   */
  public boolean isTwoScalePanels() {
    return controls.isTwoScalePanels();
  }

  /**
   * Sets flag whether two scale panels are supplied.
   */
  public void setTwoScalePanels(boolean twoScalePanels) {
    controls.setTwoScalePanels(twoScalePanels);

    if (twoScalePanels) {
      // first panel sets x scale, second y scale
      setScaleX(true);
      setScaleY(false);

      controls.addVectorListener2(new VectorListener() {
        public void valueChanged(VectorEvent evt) {
          screen.changeYScale(controls.getScaleChange2());
        }
      });
    }
  }

}
