package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.beans.*;
import java.io.*;
import physbeans.model.*;

/**
 * Bean for input of a real number via a JSlider<br>
 * slider characteristic can be linear or logarithmic<br>
 * slider can send PropertyChange events either continuously (during drag) or
 *   only after drag stopped<br>
 * internal model consists of three doubles minVal <= value <= maxVal<br>
 * changes to model parameters leading to an inconsistent state are not
 *  simply rejected, but lead to a rearrangement of all three values.
 *  This should allow for simpler changes of the internal model.
 */
public class Slider extends PhysicsPanel implements Serializable {
  
  protected BoundedRealModel model; // Slider model: (minVal, value, maxVal)
  protected boolean isLogarithmic;
  protected boolean fireContinuously;
  protected boolean vertical;
  protected boolean paintTicks;
  protected boolean marked;
  
  protected JSlider mySlider;
  protected int nTicks;   // number of JSlider steps
  
  protected boolean inhibitEvents;
  
  /** Creates a new instance of Slider */
  public Slider() {
    super();
    setLayout(new GridBagLayout());  // centers the slider
    setFocusable(true);
    model = new BoundedRealModel();
    nTicks = 10000;
    
    fireContinuously = false;
    isLogarithmic = false;
    inhibitEvents = false;
    paintTicks = true;
    vertical = false;
    marked = false;
    
    // JSlider range from 0 to nTicks
    mySlider = new JSlider(JSlider.HORIZONTAL, 0, nTicks, nTicks/2);
    mySlider.setMajorTickSpacing(nTicks/10);
    mySlider.setMinorTickSpacing(nTicks/50);
    mySlider.setPaintTicks(true);
    mySlider.setBorder(new EmptyBorder(1, 1, 1, 1));
    mySlider.addChangeListener(new MyJSliderListener());
    mySlider.addFocusListener(new MyJSliderFocusListener());
    mySlider.setOpaque(false);  // doesn't affect the tick marks!
    add(mySlider);
  }
  
  /** returns value corresponding to current slider position */
  protected double getJSliderValue() {
    double val = (double) mySlider.getValue(); // between 0 and nTicks
    val /= nTicks; // between 0 and 1
    double max = model.getMaximum();
    double min = model.getMinimum();
    
    if (isLogarithmic) {
      val = Math.exp(val*Math.log(max) + (1 - val)*Math.log(min));
    } else {
      val = val*max + (1 - val)*min;
    }
    return val;
  }
  
  /** sets position of the JSlider */
  protected void setJSliderValue(double val) {
    // check input value
    double max = model.getMaximum();
    double min = model.getMinimum();
    val = Math.max(Math.min(val, max), min);
    
    // convert value to scrollbar interval 0 .. nTicks
    double sliderValue;
    if (isLogarithmic) {
      sliderValue = nTicks *
          (Math.log(val) - Math.log(min))/(Math.log(max) - Math.log(min));
    } else {
      sliderValue = nTicks * (val - min)/(max - min);
    }
    
    inhibitEvents = true;
    mySlider.setValue((int) Math.round(sliderValue));
    inhibitEvents = false;
  }
  
  /**
   * Returns the current value.
   */
  public double getValue() {
    return model.getValue();
  }
  
  /**
   * Sets the current value and adjusts the JSlider accordingly.<br>
   * fires a property change event.
   */
  public void setValue(double val) {
    setValueInternally(val);
    firePropertyChange("value", null, new Double(val));
  }
  
  /**
   * Sets the current value and adjusts the JSlider accordingly
   * without firing an event
   */
  void setValueInternally(double val) {
    // package access to allow TextSlider and VectorSlider to use it
    model.setValue(val);
    setJSliderValue(val);
  }
  
  /**
   * True if events are sent during the drag.
   */
  public boolean isFireContinuously() {
    return fireContinuously;
  }
  
  /** sets value of fireContinuously flag */
  public void setFireContinuously(boolean flag) {
    fireContinuously = flag;
  }
  
  /**  true if slider characteristic is logarithmic */
  public boolean isLogarithmic() {
    return isLogarithmic;
  }
  
  /** sets slider characteristic (linear/logarithmic) */
  public void setLogarithmic(boolean flag) {
    isLogarithmic = flag;
    
    if (isLogarithmic) {
      // check for minimum
      if (model.getMinimum() <= 0) {
        model.setMinimum(1.0);
      }
      
      // adapt slider
      setJSliderValue(getValue());
    }
  }
  
  /** returns minimal value */
  public BoundedRealModel getModel() {
    return model;
  }
  
  /** sets model<br>
   *  adjusts JSlider accordingly
   */
  public void setModel(BoundedRealModel newModel) {
    model = newModel;
    setJSliderValue(model.getValue());
  }
  
  /**
   * sets background color
   */
  public void setBackground(Color c) {
    super.setBackground(c);
    if (mySlider != null) {
      mySlider.setBackground(c);
    }
  }
  
  /**
   * sets foreground color
   */
  public void setForeground(Color c) {
    super.setForeground(c);
    if (mySlider != null) {
      mySlider.setForeground(c);
    }
  }
  
  /**
   * Gets value of inverted flag.<br>
   *  If true, maximal slider value is to the left
   */
  public boolean getInverted() {
    return mySlider.getInverted();
  }
  
  /**
   * Sets value of inverted flag.<br>
   *  True means maximal slider value is to the left
   */
  public void setInverted(boolean b) {
    mySlider.setInverted(b);
  }
  
  /**
   * Gets value of paintTicks flag.<br>
   * If true, tick marks are to be painted.
   */
  public boolean getPaintTicks() {
    return paintTicks;
  }
  
  /**
   * Sets value of paintTicks flag.<br>
   * If true, tick marks are to be painted.
   */
  public void setPaintTicks(boolean b) {
    paintTicks = b;
    mySlider.setPaintTicks(b);
  }
  
  /**
   * Gets value of vertical flag.<br>
   * If true, the slider is oriented vertically.
   */
  public boolean isVertical() {
    return vertical;
  }
  
  /**
   * Sets value of vertical flag.<br>
   * If true, the slider is oriented vertically.
   */
  public void setVertical(boolean b) {
    vertical = b;
    int orientation = b ? JSlider.VERTICAL : JSlider.HORIZONTAL;
    mySlider.setOrientation(orientation);
  }
  
  /**
   * Gets value of marked flag.<br>
   * If true, a small border is drawn around the slider
   */
  public boolean isMarked() {
    return marked;
  }
  
  /**
   * Sets value of marked flag.<br>
   * If true, a small border is drawn around the slider
   */
  public void setMarked(boolean b) {
    if (b == marked) {
      return;    // no change
    }
    marked = b;
    
    if (marked) {
      mySlider.setBorder(new LineBorder(new Color(122, 138, 153)));
    } else {
      mySlider.setBorder(new EmptyBorder(1, 1, 1, 1));
    }
  }
  
  /**
   * Listener to JSlider events.<br>
   *  Sets value and fires PropertyChange event.
   */
  protected class MyJSliderListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      if (!inhibitEvents && (fireContinuously || !mySlider.getValueIsAdjusting())) {
        // get new value from JSlider
        double val = getJSliderValue();
        model.setValue(val);
        
        // inform listeners
        firePropertyChange("value", null, new Double(val));
      }
    }
  }
  
  /**
   * Listener that transfers focus from the JSlider to the envelopping Slider
   */
  protected class MyJSliderFocusListener implements FocusListener {
    public void focusGained(FocusEvent e) {
      requestFocusInWindow();
    }
    
    public void focusLost(FocusEvent e) {
    }
  }
  
}