package physbeans.inout;

import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import physbeans.event.*;

import physbeans.model.*;

/**
 * clock to drive a simulation or animation<br>
 * represented by buttons (play/pause, rewind) and a scrollbar (speed)<br>
 * sends NextStepEvents (more or less) regularly
 */
public class Timer extends PhysicsPanel implements Runnable, Serializable {

  protected long delay;
  protected double t0;
  protected double te;
  protected double dt;
  protected double t;
  /** state of the playOrPause button */
  protected boolean paused;
  /** thread for running the timer */
  transient protected Thread thr;
  /** factor for setting the minimal and maximal delay times */
  protected final int delaySpread = 10;
  /** minimal argument of sleep() call */
  protected final int minSleepTime = 1;
  protected final double eps = 1e-10;   // relative accuracy of time comparisons
  // components
  protected Slider speeder; // sets the animation speed
  protected JPanel buttonPanel;
  protected JButton playOrPause;
  protected JButton rewind;
  protected JLabel explanation;
  transient protected ImageIcon playIcon;
  transient protected ImageIcon pauseIcon;
  transient protected ImageIcon rewindIcon;

  /**
   * create timer with standard values
   */
  public Timer() {
    super();
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBordered(true);

    delay = 50L;
    t0 = 0.0;
    dt = 0.1;
    te = Double.POSITIVE_INFINITY;
    t = t0;
    paused = true;

    int buttonWidth = 60;
    int buttonHeight = 20;

    getIcons();

    playOrPause = new JButton(playIcon);
    playOrPause.setPreferredSize(new Dimension(buttonWidth, buttonHeight));
    playOrPause.setOpaque(false);
    playOrPause.addActionListener(new MyPlayListener());

    rewind = new JButton(rewindIcon);
    rewind.setPreferredSize(new Dimension(buttonWidth, buttonHeight));
    rewind.setOpaque(false);
    rewind.addActionListener(new MyRewindListener());

    buttonPanel = new JPanel();
    buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    buttonPanel.setOpaque(false);
    buttonPanel.setAlignmentX(0.5f);

    explanation = new JLabel("<< Slower | Faster >>");
    explanation.setAlignmentX(0.5f);
    explanation.setFont(null);
    explanation.setBorder(BorderFactory.createEmptyBorder(4, 0, 4, 0));

    speeder = new Slider();
    speeder.setLogarithmic(true);
    speeder.setInverted(true);
    double d = (double) delay;
    speeder.setModel(new BoundedRealModel(d / delaySpread, d, d * delaySpread));
    speeder.addPropertyChangeListener(new MySliderListener());
    speeder.setAlignmentX(0.5f);

    // buildGUI
    buttonPanel.add(Box.createHorizontalGlue());
    buttonPanel.add(playOrPause);
    buttonPanel.add(Box.createHorizontalGlue());
    buttonPanel.add(rewind);
    buttonPanel.add(Box.createHorizontalGlue());
    add(buttonPanel);
    add(explanation);
    add(speeder);
  }

  /**
   * Gets standard time in ms between NextStep events.<br>
   * This is the value at the middle slider position, not the current value.
   */
  public long getDelay() {
    double d = speeder.getModel().getValue();
    return Math.round(d);
  }

  /**
   * Sets standard time in ms between NextStep events.<br>
   *  Slider is set to middle position.<br>
   *  Changes minimum and maximum of slider values accordingly.
   */
  public void setDelay(long delay) {
    double d = (double) delay;
    speeder.setModel(new BoundedRealModel(d / delaySpread, d, d * delaySpread));
    this.delay = delay;
  }

  /**
   * Gets string describing meaning of slider.
   */
  public String getExplanation() {
    return explanation.getText();
  }

  /**
   * Sets string describing meaning of slider.
   */
  public void setExplanation(String str) {
    explanation.setText(str);
  }

  /**
   * Gets value of current simulated time.
   * It is incremented by timeIncrement at each NextStep event.
   */
  public double getTime() {
    return t;
  }

  /**
   * Gets value of simulated time at beginning or after restart.
   */
  public double getStartTime() {
    return t0;
  }

  /**
   * Gets value of simulated time, at which the timer stops.<br>
   * Default value is infinity, which means that the timer doesn't stop
   * automatically.
   * @return Value of property stopTime.
   */
  public double getStopTime() {
    return this.te;
  }

  /**
   * Sets value of simulated time, at which the timer stops.<br>
   * Default value is infinity, which means that the timer doesn't stop
   * automatically.
   */
  public void setStopTime(double te) {
    this.te = te;
  }

  /**
   * Sets value of simulated time at beginning or after restart.<br>
   * If the start time is set without a restart the current time jumps to the
   * next value that is reachable with the new settings of start time and time
   * increment.
   */
  public void setStartTime(double t0) {
    this.t0 = t0;
    double tneu = dt * Math.ceil((t - t0) / dt) + t0;
    t = Math.max(tneu, t0);  // adjust current time if necessary
  }

  /**
   * Gets difference of simulated time values between two NextStep events.
   */
  public double getTimeIncrement() {
    return dt;
  }

  /**
   * Sets difference of simulated time values between two NextStep events.
   */
  public void setTimeIncrement(double offset) {
    dt = offset;
  }

  /**
   * sets foreground color
   */
  public void setForeground(Color c) {
    super.setForeground(c);
    if (buttonPanel != null) {
      buttonPanel.setForeground(c);
    }
    if (playOrPause != null) {
      playOrPause.setForeground(c);
    }
    if (rewind != null) {
      rewind.setForeground(c);
    }
    if (speeder != null) {
      speeder.setForeground(c);
    }
  }

  /**
   * run the clock
   */
  public void run() {
    long sleepTime;
    while (Thread.currentThread() == thr) {

      // send next "clock tick"
      long curTime = System.currentTimeMillis();
      if (t <= te * (1 + eps)) {
        fireNextStepEvent(new NextStepEvent(this, t));

        // adjust the sleep time by the time for event delivery
        t += dt;
        sleepTime = Math.max(minSleepTime, delay - (System.currentTimeMillis()
                - curTime));
        try {
          thr.sleep(sleepTime);
        } catch (InterruptedException e) {
          break;
        }
      } else {
        changeTimerState();
        fireStopEvent(new StopEvent(this, te));
      }
    }
  }

  /**
   * start Timer thread. <br>
   * create new one, if necessary
   */
  public void startTimer() {
    if (paused) {
      // do nothing
    } else {
      //Start animating!
      if (thr == null) {
        thr = new Thread(this);
      }
      thr.start();
    }
  }

  /**
   * Rewinds timer thread and fires restart event.
   */
  public void restart() {
    thr = null;
    t = t0;
    paused = true; // don't send new Next events
    playOrPause.setIcon(playIcon);

    // inform listeners to restart
    fireRestartEvent(new RestartEvent(this, t0));
  }

  /**
   * find and load icons
   */
  protected void getIcons() {
    // basic problem:
    //    as part of an applet I can't use the image file path directly
    //    as a bean (not yet part of an applet) I can't use applet's getCodebase
    // so I have to use a ClassLoader explicitly
    //  -> basic URL starts at top of package hierarchy
    //  -> include physbeans in path!
    ClassLoader myCL = getClass().getClassLoader();
    URL playURL = myCL.getResource("physbeans/resources/play.gif");
    if (playURL != null) {
      playIcon = new ImageIcon(playURL);
    }
    URL pauseURL = myCL.getResource("physbeans/resources/pause.gif");
    if (pauseURL != null) {
      pauseIcon = new ImageIcon(pauseURL);
    }
    URL rewindURL = myCL.getResource("physbeans/resources/rewind.gif");
    if (rewindURL != null) {
      rewindIcon = new ImageIcon(rewindURL);
    }
  }

  /**
   * change current delay time according to slider
   */
  protected void changeCurrentDelayTime() {
    delay = Math.round(speeder.getValue());
  }

  /**
   * Switch between play and pause.<br>
   * This is usually called by the play button listener after a user click, but
   * it can be called manually, e.g. for test purposes or to start a simulation
   * with the timer already running.
   */
  public void changeTimerState() {
    if (paused) { // resume thread
      playOrPause.setIcon(pauseIcon);
      paused = false;
      startTimer();
    } else { // suspend thread
      playOrPause.setIcon(playIcon);
      paused = true;
      thr = null;
    }
  }
  // support TimerEvents
  protected transient TimerSupport timerSupport = new TimerSupport();

  /** registers a listener to Timer Events */
  public synchronized void addTimerListener(TimerListener l) {
    timerSupport.addTimerListener(l);
  }

  /** de-registers a listener to Timer Events */
  public synchronized void removeTimerListener(TimerListener l) {
    timerSupport.removeTimerListener(l);
  }

  /** fires a NextStepEvent */
  public void fireNextStepEvent(NextStepEvent e) {
    timerSupport.fireNextStepEvent(e);
  }

  /** fires a RestartEvent */
  public void fireRestartEvent(RestartEvent e) {
    timerSupport.fireRestartEvent(e);
  }
  // support StopEvents
  protected transient TimerStopSupport timerStopSupport = new TimerStopSupport();

  /** registers a listener to TimerStop Events */
  public synchronized void addTimerStopListener(TimerStopListener l) {
    timerStopSupport.addTimerStopListener(l);
  }

  /** de-registers a listener to TimerStop Events */
  public synchronized void removeTimerStopListener(TimerStopListener l) {
    timerStopSupport.removeTimerStopListener(l);
  }

  /** fires a StopEvent */
  public void fireStopEvent(StopEvent e) {
    timerStopSupport.fireStopEvent(e);
  }

  // listener to slider
  protected class MySliderListener implements PropertyChangeListener {

    public void propertyChange(PropertyChangeEvent e) {
      changeCurrentDelayTime();
    }
  }

  // listener to Play/Pause button
  protected class MyPlayListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      changeTimerState();
    }
  }

  // listener to rewind button
  protected class MyRewindListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      restart();
    }
  }
}
