package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.beans.*;
import java.io.*;
import physbeans.math.*;

/**
 * Bean for output of a vector of real numbers using
 * a row or a column of number text fields.<br>
 * All values have the same output precision.<br>
 * The text fields are surrounded by a description text and a unit.
 */
public class VectorTextField extends PhysicsPanel implements Serializable {
  
  protected NumericTextField[] textfield;
  protected boolean horizontal;
  protected DVector values;
  protected int columns;
  protected int precision;
  protected String[] description;
  protected String[] unit;
  protected int pad;
  
  /**
   * constructs a default VectorTextField
   */
  public VectorTextField() {
    horizontal = false;
    columns = 5;
    precision = 3;
    description = new String[] {"x =", "y ="};
    unit = new String[] {"m", "m"};
    pad = 1;
    setDimension(2);
  }
  
  /**
   * Gets number of text fields.
   */
  public int getDimension() {
    return textfield.length;
  }
  
  /**
   * Sets number of text fields.
   */
  public void setDimension(int dimension) {
    values = new DVector(dimension);
    textfield = new NumericTextField[dimension];
    for (int i = 0; i < getDimension(); i++) {
      textfield[i] = new NumericTextField();
    }
    setColumns(columns);
    setPrecision(precision);
    setDescription(description);
    setUnit(unit);
    removeAll();
    arrangeTextFields();
  }
  
  
  /**
   * Getter for property horizontal.
   * @return Value of property horizontal.
   */
  public boolean isHorizontal() {
    return this.horizontal;
  }
  
  /**
   * Setter for property horizontal.
   * @param horizontal New value of property horizontal.
   */
  public void setHorizontal(boolean horizontal) {
    this.horizontal = horizontal;
    arrangeTextFields();
  }
  
  // auxiliary functions
  
  // create gui for the text fields
  protected void arrangeTextFields() {
    int n = getDimension();
    if (horizontal) {
      setLayout(new GridLayout(1, n, pad, 3));
    } else {
      setLayout(new GridLayout(n, 1, 3, pad));
    }
    
    for (int i = 0; i < n; i++) {
      add(textfield[i]);
    }
  }
  
  /**
   * Getter for property value.
   * @return Value of property value.
   */
  public DVector getValue() {
    return values;
  }
  
  /**
   * Setter for property value.
   * If parameter vector is too short, only the first values are set.
   * If the parameter vector is too long, its last values are ignored.
   * @param v New value of property value.
   */
  public void setValue(DVector v) {
    // explicit field values is needed to hold initial values
    values = v;
    
    int n = Math.min(v.getDimension(), textfield.length);
    for (int i = 0; i < n; i++) {
      textfield[i].setValue(v.get(i));
    }
  }
  
  /**
   * Gets common precision of the text fields.
   */
  public int getPrecision() {
    return precision;
  }
  
  /**
   * Sets common precision of the text fields.
   */
  public void setPrecision(int prec) {
    precision = prec;
    for (int i = 0; i < textfield.length; i++) {
      textfield[i].setPrecision(precision);
    }
  }
  
  /**
   * Gets common number of columns of the text fields.
   */
  public int getColumns() {
    return columns;
  }
  
  /**
   * Sets common number of columns of the text fields.
   */
  public void setColumns(int cols) {
    columns = cols;
    for (int i = 0; i < textfield.length; i++) {
      textfield[i].setColumns(columns);
    }
  }
  
  /**
   * Indexed getter for property description.
   * @param index Index of the property.
   * @return Value of the property at <CODE>index</CODE>.
   */
  public String getDescription(int index) {
    return description[index];
  }
  
  /**
   * Getter for property description.
   * @return Value of property description.
   */
  public String[] getDescription() {
    return description;
  }
  
  /**
   * Indexed setter for property description.
   * @param i Index of the property.
   * @param desc New value of the property at <CODE>index</CODE>.
   */
  public void setDescription(int i, String desc) {
    description[i] = desc;
    textfield[i].setDescription(desc);
  }
  
  /**
   * Setter for property description.
   * @param desc New value of property description.
   */
  public void setDescription(String[] desc) {
    description = desc;
    int n = Math.min(desc.length, textfield.length);
    for (int i = 0; i < n; i++) {
      textfield[i].setDescription(desc[i]);
    }
  }
  
  /**
   * Indexed getter for property unit.
   * @param index Index of the property.
   * @return Value of the property at <CODE>index</CODE>.
   */
  public String getUnit(int index) {
    return unit[index];
  }
  
  /**
   * Getter for property unit.
   * @return Value of property unit.
   */
  public String[] getUnit() {
    return unit;
  }
  
  /**
   * Indexed setter for property unit.
   * @param i Index of the property.
   * @param u New value of the property at <CODE>index</CODE>.
   */
  public void setUnit(int i, String u) {
    unit[i] = u;
    textfield[i].setUnit(u);
  }
  
  /**
   * Setter for property unit.
   * @param u New value of property unit.
   */
  public void setUnit(String[] u) {
    unit = u;
    int n = Math.min(u.length, textfield.length);
    for (int i = 0; i < n; i++) {
      textfield[i].setUnit(u[i]);
    }
  }
  
  /**
   * Gets width of padding around each text field.
   */
  public int getPadding() {
    return pad;
  }
  
  /**
   * Sets width of  padding around each text field.
   */
  public void setPadding(int pad) {
    this.pad = pad;
    arrangeTextFields();
  }
}
