package physbeans.phys;

import physbeans.math.*;
import physbeans.model.*;

/**
 * Vector field describing the electrostatic field of an ideal dipole.<br>
 * + This is the limit of two opposite point charges for vanishing distance a and
 * infinite charge q, such that q*a =: d (the dipole moment) remains constant.<br>
 * Geometry is fixed: x axis through the charges, 0 in the center.
 */
public class AsymptoticDipoleField extends VectorField {
  
  /** dipole moment of the dipole in C*m */
  protected double dipoleMoment;
  protected boolean scalarfieldIsPotential;
  
  /** Creates a new instance of DipoleField */
  public AsymptoticDipoleField() {
    super(2,2);
    dipoleMoment = 1.0;
  }
  
  /**
   * Gets vector value at given coordinates.
   */
  public DVector getVectorValue(DVector v) {
    // implementation using vector values
    // if too slow, use coordinates directly
    
    // positive charge to the left
    double x = v.get(0);
    double y = v.get(1);
    double r = Math.hypot(x,y);
    double r2 = r*r;
    
    DVector result = new DVector(2.0*x*x - y*y, 3.0*x*y);
    result.timesEquals(dipoleMoment/(4.0*Math.PI*Constants.EPSILON0*r*r2*r2));
    
    return result;
  }
  
  /**
   * Returns a scalar value associated with the vector field
   *   at the given coordinates.<br>
   * This is either the absolute value or the field 's potential depending
   * on the value of flag scalarfieldIsPotential.
   */
  public double getScalarValue(DVector v) {
    if (scalarfieldIsPotential) {
      return getPotential(v);
    } else {
      return getAbsoluteValue(v);
    }
  }
  
  /**
   * Returns the absolute value at the given coordinates.<br>
   */
  public double getAbsoluteValue(DVector v) {
    return getVectorValue(v).norm();
  }
  
  /**
   * Gets electric potential at given coordinates v.
   */
  public double getPotential(DVector v) {
    // positive charge to the left (r1)
    double x = v.get(0);
    double r = v.norm();
    double r3 = r*r*r;
    
    double result = dipoleMoment/(4*Math.PI*Constants.EPSILON0) * (x/r3);
    return result;
  }
  
  /**
   * Gets dipole moment of the dipole in C*m.
   */
  public double getDipoleMoment() {
    return this.dipoleMoment;
  }
  
  /**
   * Sets dipole moment of the dipole in C*m.
   */
  public void setDipoleMoment(double dipoleMoment) {
    this.dipoleMoment = dipoleMoment;
    trigger();
  }
  
  /**
   * Gets value of flag scalarfieldIsPotential.
   * If true, the associated scalarfeld is the potential of the vector field,
   * otherwise it is its absolute value.
   */
  public boolean getScalarfieldIsPotential() {
    return this.scalarfieldIsPotential;
  }
  
  /**
   * Sets value of flag scalarfieldIsPotential.
   * If true, the associated scalarfeld is the potential of the vector field,
   * otherwise it is its absolute value.
   */
  public void setScalarfieldIsPotential(boolean scalarfieldIsPotential) {
    this.scalarfieldIsPotential = scalarfieldIsPotential;
    trigger();
  }
}
