package physbeans.phys;

import static java.lang.Math.*;
import physbeans.func.*;
import physbeans.math.*;

/**
 * Collision of two particles moving in one dimension<br>
 * The collision is partly inelastic, the energy loss is described by the
 * elaticity alpha, the fraction of the initial CMS energy after the
 * collision.<br>
 * The particles are solid balls with radii that vary with the mass given by a
 * constant density.<br>
 * The standard result vector has length 3, it contains the current position of
 * the two particles and of the center of mass (as third element).
 */
public class Collision1D extends Generic1dFunction {

  DVector m;
  DVector v;
  DVector x0;
  double al;
  double rho;

  public Collision1D() {
    m = new DVector(1.0, 1.0);
    v = new DVector(1.0, -1.0);
    x0 = new DVector(-1.0, 1.0);
    al = 1.0;
    rho = 10.0;
  }

  @Override
  public DVector computeFunction(double t) {
    // returns the positions of the two particles at time t and the position
    // of the center-of-mass 
    DVector x;

    double tS = getTimeOfCollision();
    DVector xS = v.times(tS).plus(x0);

    if (t <= tS || tS <= 0) {
      // before the collision or if the collision happened before t = 0
      x = v.times(t).plus(x0);
    } else {           // after the collision
      DVector vp = getVelocitiesAfterCollision();
      x = vp.times(t - tS).plus(xS);
    }

    double xCms = m.times(x0) / (m.get(0) + m.get(1)) + getVelocityCms() * t;
    return new DVector(x.get(0), x.get(1), xCms);
  }

  /**
   * Gets the velocities after the collision.
   */
  public DVector getVelocitiesAfterCollision() {
    double vCms = getVelocityCms();
    double b = sqrt(al) * (v.get(0) - v.get(1)) / (m.get(0) + m.get(1));
    double u1 = vCms - b * m.get(1);
    double u2 = vCms + b * m.get(0);
    return new DVector(u1, u2);
  }

  /**
   * Gets the velocity of the center-of-mass.
   */
  public double getVelocityCms() {
    return m.times(v) / (m.get(0) + m.get(1));
  }

  /**
   * Gets the masses of the two colloding bodies.
   */
  public DVector getMasses() {
    return m;
  }

  /**
   * Sets the masses of the two colloding bodies.
   */
  public void setMasses(DVector m) {
    this.m = m;
    inform();
  }

  /**
   * Gets the velocities of the two colloding bodies before the collision.
   */
  public DVector getInitialVelocities() {
    return v;
  }

  /**
   * Sets the velocities of the two colloding bodies before the collision.
   */
  public void setInitialVelocities(DVector v) {
    this.v = v;
    inform();
  }

  /**
   * Gets the initial positions of the two colloding bodies.
   */
  public DVector getInitialPositions() {
    return x0;
  }

  /**
   * Sets the initial positions of the two colloding bodies.
   */
  public void setInitialPositions(DVector x0) {
    this.x0 = x0;
    inform();
  }

  /**
   * Gets the elasticity alpha of the collision.<br>
   * alpha is the fraction of the total kinetic energies after and before the
   * collision, as seen in the center-or-mass system.
   */
  public double getElasticity() {
    return al;
  }

  /**
   * Sets the elasticity alpha of the collision.<br>
   * alpha is the fraction of the total kinetic energies after and before the
   * collision, as seen in the center-or-mass system.
   */
  public void setElasticity(double al) {
    this.al = al;
    inform();
  }

  /**
   * Gets the density of the particles.<br>
   * This is used to compute their radii assuming homogeneous balls.
   */
  public double getDensity() {
    return rho;
  }

  /**
   * Sets the density of the particles.<br>
   * This is used to compute their radii assuming homogeneous balls.
   */
  public void setDensity(double rho) {
    this.rho = rho;
    inform();
  }

  /**
   * Gets the radii of the particles, as given by their density and mass.
   */
  public DVector getRadii() {
    int n = m.getDimension();
    DVector ri = new DVector(n);
    for (int i = 0; i < n; i++) {
      double r = cbrt(3 * m.get(i) / (4 * PI * rho));
      ri.set(i, r);
    }
    return ri;
  }

  /**
   * Gets the time when the two particles collide.
   */
  public double getTimeOfCollision() {
    DVector ri = getRadii();
    double d = ri.get(0) + ri.get(1);
    double dx0 = x0.get(1) - x0.get(0);
    double tS = (dx0 - d * Maths.sign(dx0)) / (v.get(0) - v.get(1));
    return tS;
  }

  /**
   * Gets the initial positions of the two colloding bodies and the
   * center-of-mass.<br>
   * Conveniance function to simplify routing.
   */
  public DVector getAllInitialPositions() {
    double xCms0 = m.times(x0) / (m.get(0) + m.get(1));
    return new DVector(x0.get(0), x0.get(1), xCms0);
  }

  /**
   * Gets the velocities of the two particles.
   */
  public DVector getVelocities() {
    double tS = getTimeOfCollision();
    double t = getInputValues(0);
    if (t <= tS || tS <= 0) {
      // before the collision or if the collision happened before t = 0
      return v;
    } else {           // after the collision
      return getVelocitiesAfterCollision();
    }
  }
  
  /**
   * Gets the kinetic energies of the two particles.
   */
  public DVector getEnergies() {
    DVector v = getVelocities();
    DVector E = m.arrayTimes(v).arrayTimes(v).times(0.5);
    return E;
  }
}
