package physbeans.phys;

import physbeans.math.*;
import static java.lang.Math.*;
import static physbeans.math.SpecialPolynomials.*;
import static physbeans.phys.Constants.*;

/**
 * Creates an arbitrary superposition of eigenfunctions of the 2d Coulomb
 * Hamiltonian as initial values on a 2d complex grid.
 * The eigenfunctions are defined by two numbers:
 *      n    -> E_n = -&gamma;^2/(2 (n - 1/2)^2)
 *      m    -> L_z = hbar m
 * For a given n one has 2*n-1 values of m:
 *        m = -(n-1), -(n-2), .., 0, 1, .., (n-1) .
 */
public class Coulomb2DEigenfunction extends ComplexGridInitializer {

  protected CMatrix c;
  protected double Z;
  protected int N;              // maximal value of n
  protected double[][] K;
  protected int n0, m0;

  public Coulomb2DEigenfunction() {
    // layout of c_ij: cf. setMatrix
    N = 2;
    c = new CMatrix(new double[]{0.0, 0.5, 0.5, 0.5, 0.0, 0.5},
            new double[]{0, 0, 0, 0, 0, 0}, 2);
    Z = 1.0;
    K = new double[][]{};                   // normalisation constants
    n0 = 1;
    m0 = 0;
  }

  public void setGrid() {
    if (field == null) {
      return;
    }

    if (K.length < N) {                  // lazy initialisation
      computeNormalisationConstants(N);
    }

    double[][] gridR = field.getGridReal();
    double[][] gridI = field.getGridImag();

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bwX = field.getBoundaryWidthX();
    int bwY = field.getBoundaryWidthY();

    clearBoundaries();

    double r0 = (4 * PI * HBAR * HBAR * EPSILON0) / (Z * ME * E0 * E0);
    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = bwY; j < ny - bwY; j++) {
        DVector xVec = field.getWorldFromIndex(i, j);
        double r = hypot(xVec.x(), xVec.y());
        double phi = atan2(xVec.y(), xVec.x());

        gridR[j][i] = 0.0;
        gridI[j][i] = 0.0;
        for (int n = 1; n <= N; n++) {
          double kappa = 1 / (n - 0.5);
          for (int m = -(n - 1); m <= n - 1; m++) {
            int l = abs(m);
            double x = 2 * kappa * r / r0;
            double g = K[n - 1][l] * exp(-0.5 * x) * pow(x, l) * La(n - l
                    - 1, 2 * l, x);
            double cphi = cos(m * phi);
            double sphi = sin(m * phi);

            Complex c_nm = c.get(n - 1, m + N - 1);
            double cr = c_nm.real();
            double ci = c_nm.imag();

            gridR[j][i] += g * (cr * cphi - ci * sphi);
            gridI[j][i] += g * (ci * cphi + cr * sphi);
          }
        }
      }
    }

    normalizeField();
  }

  /**
   * Sets complex matrix of coefficients c<sub>i,j</sub>.<br>
   * Layout (n,m) of coefficients e.g. for N = 4
   *    -  -  -  0  -  -  -    (n = 1)
   *    -  - -1  0  1  -  -    (n = 2)
   *    - -2 -1  0  1  2  -    (n = 3)
   *   -3 -2 -1  0  1  2  3    (n = 4)
   */
  public void setMatrix(CMatrix c) {
    this.c = c;
    N = c.getColumnDimension();
    updateField();
  }

  /**
   * Gets complex matrix of coefficients c<sub>i,j</sub>.
   */
  public CMatrix getMatrix() {
    return c;
  }

  /**
   * Sets the central charge in multiples of e.
   */
  public void setZ(double Z) {
    this.Z = Z;
    updateField();
  }

  /**
   * Gets the central charge in multiples of e.
   */
  public double getZ() {
    return Z;
  }

  /**
   * Sets number of energy levels.
   * Default entries of new coefficient matrix:
   * all coefficients with m = 0 are equal, all with m != 0 are 0.
   */
  public void setNEnergyLevels(int N) {
    this.N = N;

    c = new CMatrix(new Matrix(N, 2 * N - 1), new Matrix(N, 2 * N - 1));
    double val = 1.0 / sqrt(N);
    for (int i = 0; i < N; i++) {
      c.set(i, N - 1, val);     // m = 0 values
    }

    updateField();
  }

  /**
   * Gets number of energy eigenfunctions.
   */
  public int getNEnergyLevels() {
    return N;
  }

  /**
   * Sets coefficients to a single eigenfunction given by n and m.
   */
  public void setSimpleEigenfunction(int n, int m) {
    N = max(n, N);
    c = new CMatrix(N, 2 * N - 1);
    c.set(n - 1, m + N - 1, new Complex(1.0));
    updateField();
  }

  /**
   * Sets coefficients to a single eigenfunction given by integer vector (n, m).
   */
  public void setSimpleEigenfunction(IVector ivec) {
    setSimpleEigenfunction(ivec.get(0), ivec.get(1));
  }

  /**
   * Sets coefficient for given n and m to the given complex value.
   */
  public void setCoefficient(int n, int m, Complex val) {
    if (abs(m) < n) {
      c.set(n - 1, m + N - 1, val);
      updateField();
    }
  }

  /**
   * Sets coefficient for given n and m to the given real value.
   */
  public void setCoefficient(int n, int m, double val) {
    if (abs(m) < n) {
      c.set(n - 1, m + N - 1, val);
      updateField();
    }
  }

  /**
   * Sets all coefficients for a given n using a real vector mVals ordered
   * by -(n-1), -(n-2), .., -1, 0, 1, ..., (n-2), (n-1).
   */
  public void setCoefficientsWithNFixed(int n, DVector mVals) {
    if (mVals.getDimension() == 2 * n - 1) {
      for (int m = -(n - 1); m <= (n - 1); m++) {
        c.set(n - 1, m + N - 1, mVals.get(m + n - 1));
      }
      updateField();
    }
  }

  /**
   * Sets all coefficients for a given m using a real vector nVals.
   */
  public void setCoefficientsWithMFixed(int m, DVector nVals) {
    int nMax = nVals.getDimension();
    c = new CMatrix(nMax, 2 * nMax + 1);

    for (int n = 1; n <= nMax; n++) {
      c.set(n - 1, m + N - 1, nVals.get(n - 1));
    }
    updateField();

  }

  /**
   * Computes the normalisation constants of the eigenfunctions up to n = max.
   */
  protected void computeNormalisationConstants(int max) {
    // K_n,l = K[n-1][l]
    K = new double[max][max];
    double r0 = (4 * PI * HBAR * HBAR * EPSILON0) / (Z * ME * E0 * E0);
    double fac = 1 / (r0 * sqrt(PI));
    for (int n = 1; n <= max; n++) {
      K[n - 1][0] = fac * pow(n - 0.5, -1.5);
      for (int l = 1; l <= (n - 1); l++) {
        K[n - 1][l] = K[n - 1][l - 1] / sqrt((n + l - 1) * (n - l));
      }
    }
  }

  /**
   * Computes the coefficient matrix for a single eigenfunction
   * given by eigenvalues n0, m0.
   */
  protected void computeSimpleMatrix() {
    N = max(n0, N);
    c = new CMatrix(N, 2 * N - 1);
    c.set(n0 - 1, m0 + N - 1, new Complex(1.0));
  }
}
