package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import physbeans.model.*;
import java.io.*;

/**
 * A physical bean describing a harmonic oscillator with coulomb friction <br>
 *    m x'' + &mu; m g sign(x') + c x = F(t)
 * <p>
 * It has the following parameters
 *  <ul>
 * <li>oscillator mass m</li>
 * <li>coulomb friction coefficient &mu;</li>
 * <li>spring constant c</li>
 * <li>gravity constant g</li>
 *  </ul>
 * An arbitrary external force F can be added via a Generic1dFunction which is
 * associated with the CoulombSpring. Details are as for the DrivenSpring.
 */
public class CoulombSpring extends SolveODEFunction implements Serializable {
  
  protected double m;
  protected double c;
  protected double mu;
  protected double g;
  protected Real1DFunction externalForceFunction;
  
  /** Creates a new instance of DrivenSpring */
  public CoulombSpring() {
    super();
    
    m  = 1.0;
    c  = 1.0;
    mu = 0.05;
    g  = 9.81;
    
    setOde(new CoulombSpringODE());
    externalForceFunction = null;
    restart();
  }
  
  /**
   * Gets mass m attached to the spring.
   */
  public double getM() {
    return this.m;
  }
  
  /**
   * Sets mass m attached to the spring.
   */
  public void setM(double m) {
    this.m = m;
    restart();
  }
  
  /**
   * Gets stiffness c of spring.
   */
  public double getC() {
    return this.c;
  }
  
  /**
   * Sets stiffness c of spring.
   */
  public void setC(double c) {
    this.c = c;
    restart();
  }
  
  /**
   * Gets Coulomb friction coefficient &mu;.
   */
  public double getMu() {
    return this.mu;
  }
  
  /**
   * Sets Coulomb friction coefficient &mu;.
   */
  public void setMu(double mu) {
    this.mu = mu;
    restart();
  }
  
  /**
   * Gets free fall acceleration g.
   */
  public double getG() {
    return this.g;
  }
  
  /**
   * Sets free fall acceleration g.
   */
  public void setG(double g) {
    this.g = g;
    restart();
  }
  
  /**
   * Gets initial position x0.
   */
  public double getX0() {
    return getInitialValue(0);
  }
  
  /**
   * Sets initial position x0.
   */
  public void setX0(double x0) {
    setInitialValue(0, x0);
  }
  
  /**
   * Gets initial velocity v0.
   */
  public double getV0() {
    return getInitialValue(1);
  }
  
  /**
   * Sets initial velocity v0.
   */
  public void setV0(double v0) {
    setInitialValue(1, v0);
  }
  
  /**
   * Gets function that computes time depending values of the external torque
   */
  public Real1DFunction getExternalForceFunction() {
    return this.externalForceFunction;
  }
  
  /**
   * Sets function that computes time depending values of the external torque
   */
  public void setExternalForceFunction(Real1DFunction externalForceFunction) {
    this.externalForceFunction = externalForceFunction;
  }
  
  class CoulombSpringODE extends ODE {
    
    /** small positive value for regularisation purposes */
    protected double epsilon = 1.0e-4;
    
    public CoulombSpringODE() {
      super(new DVector(1.0, 0.0), 0.0);
    }
    
    public DVector f(DVector x, double t) {
      DVector result = new DVector(2);
      double xt = x.get(0);   // x(t)
      double vt = x.get(1);   // v(t)
      
      // get external force as value of the externalForceFunction
      double Fext;
      if (externalForceFunction != null) {
        Fext = externalForceFunction.evaluate(t);
      } else {
        Fext = 0.0;
      }
      
      // compute forces
      double Fs = Fext - c*xt;    // spring force and external force
      double Ff;                  // friction
      if (Math.abs(vt) < epsilon) {
        // sticky (Haftreibung)
        Ff = -Maths.sign(Fs) * Math.min(Math.abs(Fs), mu*m*g);
      } else {
        // Coulomb friction
        Ff = - mu*m*g*Maths.sign(vt);
      }
      
      result.set(0, vt);              // x(0)' = x(1) = v
      result.set(1, (Fs + Ff)/m);     // x(1)' = x'' = F
      return result;
      
    }
    
  }
}
