package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Function that computes the excitation force on a linear mass-spring system
 * resulting from a harmonic basement excitation.
 * Assumes that the first mass (index 0) sits on the ground.
 * <P>Input:  time <br>
 * Output: value of ScalarField at current detector position.
 */
public class HarmonicBaseExcitation extends Generic1dFunction {

  protected LinearChainOscillator system;
  protected double uhat;
  protected double alpha;
  protected double omega;

  /** Creates a new instance of HarmonicBaseExcitation */
  public HarmonicBaseExcitation() {
    super(2);          // one input value, two output values
    uhat = 1.0;
    omega = 1.0;
    alpha = 0.0;
  }

  /**
   * Computes the value of the force that acts on the lowest mass according to
   * the harmonic movement of the basement.
   */
  public DVector computeFunction(double t) {
    DVector result = new DVector(getOutputLength());
    if (system != null) {
      double c1 = system.getSpringStiffnesses().get(0);   // lowest spring
      double b1 = system.getDamperCoefficients().get(0);  // lowest damper

      double FEhat = c1 * uhat * sqrt(1 + (b1 * b1 * omega * omega) / (c1 * c1));
      double psi = atan(b1 * omega / c1) + alpha;
      double FE = FEhat * cos(omega * t + psi);
      result.set(0, FE);

      return result;
    } else {
      result.set(0, Double.NaN);   // force is undefined
      return result;
    }
  }

  /**
   * Gets the current position of the basement.
   */
  public double getBasementExcitation() {
    return uhat * cos(omega * getInputValue() + alpha);
  }

  /**
   * Gets the current velocity of the basement.
   */
  public double getBasementVelocity() {
    return -uhat * omega * sin(omega * getInputValue() + alpha);
  }

  /**
   * Gets the LinearChainOscillator system that is excited.
   */
  public LinearChainOscillator getSystem() {
    return system;
  }

  /**
   * Sets the LinearChainOscillator system that is excited.
   */
  public void setSystem(LinearChainOscillator s) {
    system = s;
    inform();
  }

  /**
   * Gets amplitude of the harmonic function.
   */
  public double getAmplitude() {
    return this.uhat;
  }

  /**
   * Sets amplitude of the harmonic function.
   */
  public void setAmplitude(double a) {
    this.uhat = a;
    inform();
  }

  /**
   * Gets phase shift of the harmonic function.
   */
  public double getPhase() {
    return this.alpha;
  }

  /**
   * Sets phase shift of the harmonic function.
   */
  public void setPhase(double alpha) {
    this.alpha = alpha;
    inform();
  }

  /**
   * Gets angular frequency of the harmonic function.
   */
  public double getOmega() {
    return this.omega;
  }

  /**
   * Sets angular frequency of the harmonic function.
   */
  public void setOmega(double omega) {
    this.omega = omega;
    inform();
  }

  /**
   * Sets time.
   * Conveniance function to simplify routing.
   */
  public void setTime(double t) {
    setInputValue(t);
  }
}
