package physbeans.phys;

import physbeans.math.*;
import java.awt.*;

/**
 * A LineWaveSource disturbs a complete line of points in the ripple tank by
 * setting it periodically according to a harmonic function.
 * <P>Alternatively the time function can be set to a periodic pulse form, which
 * is better suited for the representation of reflection phenomena.
 * <P>In addition to the positions of the two end points and the parameters of
 * the underlying harmonic function one can define a phase difference between
 * the endpoints. The points along the line oscillate with a phase difference
 * that is constant between neighbouring points and adds up to the the given
 * total phase difference.
 * <P>The geometry of the line source can be defined either by directly specifying
 * the coordinates of the two end points, or by setting the position of the
 * center, the length of the line and ist orientation.
 */
public class LineWaveSource extends WaveSource {
  
  protected  DVector pos1, pos2;
  protected double a, om, phi;
  protected double phiDiff;
  protected boolean pulseWave;
  
  /** Creates a new instance of LineWaveSource */
  public LineWaveSource() {
    super();
    pos1 = new DVector(-1.0, 0.0);
    pos2 = new DVector(1.0, 0.0);
    a = 1.0;
    om = 1.0;
    phi = 0.0;
    phiDiff = 0.0;
    pulseWave = false;
  }
  
  public void stirTank(RippleTank tank, double t) {
    Point p1 = tank.getIndex(pos1);
    Point p2 = tank.getIndex(pos2);
    double[][] grid = tank.getGrid();
    
    if ((p1.x == -1) || (p1.y == -1) || (p2.x == -1) || (p2.y == -1)) {
      return;     // a point lies outside the grid
    }
    
    if ((p1.x == p2.x) && (p1.y == p2.y)) {
      if (pulseWave) {
        grid[p1.y][p1.x] = pulseFunction(t);
      } else {
        grid[p1.y][p1.x] = a*Math.cos(om*t + phi);
      }
      return;
    }
    
    int xStart, xEnd, yStart, yEnd;
    if (Math.abs(p1.y - p2.y) > Math.abs(p1.x - p2.x)) {  // step along y axis
      yStart = Math.min(p1.y, p2.y);
      yEnd = Math.max(p1.y, p2.y);
      if (p1.y <= p2.y) {
        xStart = p1.x;
        xEnd = p2.x;
      } else {
        xStart = p2.x;
        xEnd = p1.x;
      }
      double dPhi = phiDiff/(yEnd - yStart);
      for (int y = yStart; y <= yEnd; y++) {
        int x = xStart + (xEnd-xStart)*(y-yStart)/(yEnd-yStart);
        if (pulseWave) {
          grid[y][x] = pulseFunction(t);
        } else {
          grid[y][x] = a*Math.cos(om*t + phi + dPhi*(y - yStart));
        }
      }
    } else {    // step along x axis
      xStart = Math.min(p1.x, p2.x);
      xEnd = Math.max(p1.x, p2.x);
      if (p1.x <= p2.x) {
        yStart = p1.y;
        yEnd = p2.y;
      } else {
        yStart = p2.y;
        yEnd = p1.y;
      }
      double dPhi = phiDiff/(xEnd - xStart);
      for (int x = xStart; x <= xEnd; x++) {
        int y = yStart + (yEnd-yStart)*(x-xStart)/(xEnd-xStart);
        if (pulseWave) {
          grid[y][x] = pulseFunction(t);
        } else {
          grid[y][x] = a*Math.cos(om*t + phi + dPhi*(x - xStart));
        }
      }
    }
  }
  
  /**
   * Gets the position of the first endpoint of the line source.
   */
  public DVector getPosition1() {
    return pos1;
  }
  
  /**
   * Sets the position of the first endpoint of the line source.
   */
  public void setPosition1(DVector p) {
    pos1 = p;
  }
  
  /**
   * Gets the position of the second endpoint of the line source.
   */
  public DVector getPosition2() {
    return pos2;
  }
  
  /**
   * Sets the position of the second endpoint of the line source.
   */
  public void setPosition2(DVector p) {
    pos2 = p;
  }
  
  /**
   * Sets the x position of the first endpoint of the line source.
   * <P>Conveniance function to simplify routing.
   */
  public void setPosition1X(double x) {
    pos1.set(0,x);
  }
  
  /**
   * Sets the y position of the first endpoint of the line source.
   * <P>Conveniance function to simplify routing.
   */
  public void setPosition1Y(double y) {
    pos1.set(1,y);
  }
  
  /**
   * Sets the x position of the second endpoint of the line source.
   * <P>Conveniance function to simplify routing.
   */
  public void setPosition2X(double x) {
    pos2.set(0,x);
  }
  
  /**
   * Sets the y position of the second endpoint of the line source.
   * <P>Conveniance function to simplify routing.
   */
  public void setPosition2Y(double y) {
    pos2.set(1,y);
  }
  
  /**
   * Gets the amplitude of the source.
   */
  public double getAmplitude() {
    return a;
  }
  
  /**
   * Sets the amplitude of the source.
   */
  public void setAmplitude(double a1) {
    a = a1;
  }
  
  /**
   * Gets the frequency of the source.
   */
  public double getFrequency() {
    return om/(2*Math.PI);
  }
  
  /**
   * Sets the frequency of the source.
   */
  public void setFrequency(double f) {
    om = 2*Math.PI*f;
  }
  
  /**
   * Gets the start phase of the source.
   */
  public double getPhase() {
    return phi;
  }
  
  /**
   * Sets the start phase of the source.
   */
  public void setPhase(double p) {
    phi = p;
  }
  
  /**
   * Gets the phase difference between the two endpoints of the line source.
   * <P>If one or both endpoints lie outside the grid, the phase difference is
   * taken between the start and end point of the line inside the grid.
   */
  public double getPhaseDifference() {
    return phiDiff;
  }
  
  /**
   * Sets the phase difference between the two endpoints of the line source.
   */
  public void setPhaseDifference(double p) {
    phiDiff = p;
  }
  
  /**
   * Returns the value of the pulseWave flag.
   * <P>If true, the time function used is a periodic pulse instead of a
   * harmonic function.
   * <P>The pulse is a gaussian that repeats with the given frequency, centered
   * on the middle. Its width is given by the phase. The phase difference value
   * is ignored.
   */
  public boolean isPulseWave() {
    return pulseWave;
  }
  
  /**
   * Returns the value of the pulseWave flag.
   * <P>If true, the time function used is a periodic pulse instead of a
   * harmonic function.
   * <P>The pulse is a gaussian that repeats with the given frequency, centered
   * on the middle. Its width is given by the phase. The phase difference value
   * is ignored.
   */
  public void setPulseWave(boolean b) {
    pulseWave = b;
  }
  
  /**
   * The time function used for the pulse wave.
   */
  protected double pulseFunction(double t) {
    double phase = om*(t % (2*Math.PI/om));   // reduce to [0,2*pi]
    double arg = (phase - Math.PI)/phi;
    return a*Math.exp(-arg*arg);
  }
  
  /**
   * Get the length of the line.
   */
  public double getLength() {
    return pos2.minus(pos1).norm();
  }
  
  /**
   * Set the length of the line.
   * The orientation and the position of the center are unchanged.
   */
  public void setLength(double l) {
    setLineGeometry(getCenter(), l, getAngle());    
  }
  
  /**
   * Get the angle of the line.
   */
  public double getAngle() {
    return Math.atan2(pos2.y() - pos1.y(), pos2.x() - pos1.x());
  }
  
  /**
   * Set the angle of the line.
   * <P>It is measured against the x axis in mathematical positive direction.
   * <P>The length and the position of the center are unchanged.
   */
  public void setAngle(double al) {
    setLineGeometry(getCenter(), getLength(), al);
  }
  
  /**
   * Get the position of the line's central point.
   */
  public DVector getCenter() {
    return new DVector((pos1.x() + pos2.x())/2.0, (pos1.y() + pos2.y())/2.0);
  }
  
  /**
   * Set the position of the line's central point.
   * The orientation and the length are unchanged.
   */
  public void setCenter(DVector p0) {
    setLineGeometry(p0, getLength(), getAngle());    
  }
  
  /**
   * Set the horizontal position of the line's central point.
   * The vertical position, orientation and length are unchanged.
   * <P>Conveniance function to simplify routing.
   */
  public void setCenterX(double x) {
    setCenter(new DVector(x, (pos1.y() + pos2.y())/2.0));
  }
  
  /**
   * Set the vertical position of the line's central point.
   * The horizontal position, orientation and length are unchanged.
   * <P>Conveniance function to simplify routing.
   */
  public void setCenterY(double y) {
    setCenter(new DVector((pos1.x() + pos2.x())/2.0, y));
  }
  
  /**
   * Changes the point positions to define a line with given center, length
   * and orientation.
   */
  protected void setLineGeometry(DVector p0, double l, double al) {
    DVector v1 = new DVector(l/2*Math.cos(al), l/2*Math.sin(al));
    pos1 = p0.minus(v1);
    pos2 = p0.plus(v1);
  }
}
