package physbeans.phys;

import physbeans.math.*;
import physbeans.func.*;
import physbeans.event.*;

/**
 * A chain of linearly connected oscillators.
 * <P>It defines masses, spring stiffnesses and damping constants.
 */
public class LinearChainOscillator extends GenericCoupledSprings {
  
  protected DVector mVec, cVec, bVec;
  
  /**
   * Creates a new instance of LinearChainOscillator
   */
  public LinearChainOscillator() {
    super();
    mVec = new DVector(1.0, 1.0, 1.0);
    cVec = new DVector(1.0, 1.0, 1.0, 1.0);
    bVec = new DVector(0.0, 0.0, 0.0, 0.0);
    setMatrices();
  }
  
  /**
   * Gets the masses of the chain.
   */
  public DVector getMasses() {
    return mVec;
  }
  
  /**
   * Sets the masses of the chain.
   */
  public void setMasses(DVector v) {
    if (v.getDimension() >= 1) {
      mVec = v;
      setMatrices();
    }
  }
  
  /**
   * Sets the mass no. i<br>
   * Conveniance function to simplify routing.
   */
  public void setMass(int i, double m) {
    DVector vec = getMasses();
    vec.set(i, m);
    setMasses(vec);
  }
  
  /**
   * Gets the stiffnesses of the springs between the masses.
   * <br>The number of springs of the system has to be one larger than the
   * number f masses, but the end springs can have stiffness = zero.
   */
  public DVector getSpringStiffnesses() {
    return cVec;
  }
  
  /**
   * Gets the stiffnesses of the springs between the masses.
   * <br>The number of springs of the system has to be one larger than the
   * number f masses, but the end springs can have stiffness = zero.
   */
  public void setSpringStiffnesses(DVector v) {
    if (v.getDimension() >= 2) {
      cVec = v;
      setMatrices();
    }
  }
  
  /**
   * Sets the stiffness no. i<br>
   * Conveniance function to simplify routing.
   */
  public void setSpringStiffness(int i, double m) {
    DVector vec = getSpringStiffnesses();
    vec.set(i, m);
    setSpringStiffnesses(vec);
  }
  
  /**
   * Gets the coefficients of the dampers between the masses.
   * <br>The number of dampers of the system has to be one larger than the
   * number f masses, but the end dampers can have coefficients = zero.
   */
  public DVector getDamperCoefficients() {
    return bVec;
  }
  
  /**
   * Gets the coefficients of the dampers between the masses.
   * <br>The number of dampers of the system has to be one larger than the
   * number f masses, but the end dampers can have coefficients = zero.
   */
  public void setDamperCoefficients(DVector v) {
    if (v.getDimension() >= 2) {
      bVec = v;
      setMatrices();
    }
  }
  
  /**
   * Sets the damper coefficient no. i<br>
   * Conveniance function to simplify routing.
   */
  public void setDamperCoefficient(int i, double m) {
    DVector vec = getDamperCoefficients();
    vec.set(i, m);
    setDamperCoefficients(vec);
  }
  
  /**
   * Sets the matrices M, C and B.
   */
  protected void setMatrices() {
    // problem: the number of masses and springs can be changed in any order.
    // This can lead to intermediary inconsistent states.
    // solution: just set as many elements as the shortest vector defines.
    
    int n = mVec.getDimension();
    n = (cVec.getDimension() <= n) ? cVec.getDimension() - 1 : n;
    n = (bVec.getDimension() <= n) ? bVec.getDimension() - 1 : n;
    
    m = new Matrix(n, n);
    c = new Matrix(n, n);
    b = new Matrix(n, n);
    
    for (int i = 0; i < n; i++) {
      m.set(i, i, mVec.get(i));
    }
    
    setChainMat(c, cVec, n);
    setChainMat(b, bVec, n);
    
    if (n != getX0().getDimension()) {
      setX0(new DVector(n));      // resize output vector if necessary
    }
    
    inform();
  }
  
  /**
   * Sets the diagonal and the subdiagonal of the matrix m using the values of
   * the vector v.
   */
  protected void setChainMat(Matrix m, DVector v, int n) {
    m.set(0, 0, v.get(0) + v.get(1));
    m.set(0, 1, -v.get(1));
    for (int i = 1; i < n-1; i++) {
      m.set(i, i-1, -v.get(i));
      m.set(i, i, v.get(i) + v.get(i+1));
      m.set(i, i+1, -v.get(i+1));
    }
    m.set(n-1, n-2, -v.get(n-1));
    m.set(n-1, n-1, v.get(n-1) + v.get(n));
  }
}
