package physbeans.phys;

/**
 * A PointLightSource creates a bundle of light rays that start from the same
 * point (given by its position along and height above the optical axis).
 * The bundle is defined by the angle of the middle axis, the angular spread
 * measured against the middle axis, and the number of rays.
 */
public class PointLightSource extends LightSource {
  
  protected double midAngle;
  protected double spreadAngle;
  protected int nRays;
  
  /** Creates a default PointLightSource */
  public PointLightSource() {
    super();
    midAngle = 0.0;
    spreadAngle = Math.PI/6;
    nRays = 5;
    trigger();
  }
  
  /**
   * Gets the angle of the symmetrix axis of the light ray bundle against the
   * optical axis.
   */
  public double getMidAngle() {
    return this.midAngle;
  }
  
  /**
   * Sets the angle of the symmetrix axis of the light ray bundle against the
   * optical axis.
   */
  public void setMidAngle(double midAngle) {
    this.midAngle = midAngle;
    trigger();
  }
  
  /**
   * Gets the angular spread of the light ray bundle measured against its
   * symmetry axis.
   */
  public double getSpreadAngle() {
    return this.spreadAngle;
  }
  
  /**
   * Sets the angular spread of the light ray bundle measured against its
   * symmetry axis.
   */
  public void setSpreadAngle(double spreadAngle) {
    this.spreadAngle = spreadAngle;
    trigger();
  }
  
  /**
   * Gets the number of rays in the bundle.
   */
  public int getNRays() {
    return this.nRays;
  }
  
  /**
   * Sets the number of rays in the bundle.
   * If the number is <= 0, a minimal bundle with one ray is produced.
   */
  public void setNRays(int n) {
    nRays = Math.max(n, 1);
    trigger();
  }
  
  protected void computeRays() {
    rays = new LightRayBundle();
    if (nRays > 1) {
      double a0 = midAngle - spreadAngle;
      double da = 2*spreadAngle/(nRays - 1);
      for (int i = 0; i < nRays; i++) {
        rays.addRay(new LightRay(pos, height, a0+i*da));
      }
    } else {    // minimum: one ray
      rays.addRay(new LightRay(pos, height, midAngle));
    }
  }
}
