package physbeans.phys;

/**
 * Describes a reversible state change of a thermodynamic system.<br>
 * It is modelled as a polytrope with a given exponent n, which may be infinite
 * (for an isochore). The start point is defined by providing p and V. The final
 * point is defined by the end volume or (for isochores) by the end pressure.<br>
 * The state change can be done in a number of steps, in which case equidistant
 * intermediate values between start and end volume (or pressure for isochores)
 * are computed.
 */
public class ReversibleStateChange implements java.io.Serializable {
  
  protected ThermodynamicSystem system;
  protected double n;
  protected double pStart;
  protected double vStart;
  
  protected double vEnd;
  protected double pEnd;
  
  protected int nSteps;
  protected int nCur;      // number of current step
  
  /** Creates a new instance of ReversibleStateChange */
  public ReversibleStateChange() {
    n = 1.2;
    pStart = 1e5;       // in Pa
    vStart = 1e-3;      // in m^3
    vEnd = 1e-2;
    pEnd = 1e4;
    nSteps = 1;
    nCur = 0;
  }
  
  /**
   * Gets the system to which the state change is applied.
   * @return the thermodynamic system used.
   */
  public ThermodynamicSystem getSystem() {
    return this.system;
  }
  
  /**
   * Sets the system to which the state change is applied.
   * @param system New thermodynamic system to use.
   */
  public void setSystem(ThermodynamicSystem system) {
    this.system = system;
    restart();
  }
  
  /**
   * Gets the exponent n of the polytrope.
   */
  public double getN() {
    return this.n;
  }
  
  /**
   * Sets the exponent n of the polytrope.<br>
   * If the value is Infinity - i.e. the polytrope is a isochoric really -
   *
   */
  public void setN(double n) {
    this.n = n;
  }
  
  /**
   * Gets the start value of the pressure.
   */
  public double getStartPressure() {
    return pStart;
  }
  
  /**
   * Sets the start value of the pressure.
   */
  public void setStartPressure(double p) {
    pStart = p;
    if (system != null && p > 0.0) {
      system.setState(pStart, vStart);
    }
  }
  
  /**
   * Gets the start value of the volume.
   */
  public double getStartVolume() {
    return vStart;
  }
  
  /**
   * Sets the start value of the volume.
   */
  public void setStartVolume(double v) {
    vStart = v;
    if (system != null && v > 0.0) {
      system.setState(pStart, vStart);
    }
  }
  
  /**
   * Gets the final volume of the state change.
   */
  public double getFinalVolume() {
    return this.vEnd;
  }
  
  /**
   * Sets the final volume of the state change.
   */
  public void setFinalVolume(double vEnd) {
    if (vEnd > 0.0) {
      this.vEnd = vEnd;
    }
  }
  
  /**
   * Gets the final pressure of the state change.<br>
   * This value is used for an isochoric state change.
   */
  public double getFinalPressure() {
    return this.pEnd;
  }
  
  /**
   * Sets the final pressure of the state change.
   * This value is used for an isochoric state change.
   */
  public void setFinalPressure(double pEnd) {
    if (pEnd > 0.0) {
      this.pEnd = pEnd;
    }
  }
  
  /**
   * Gets number of steps used for the state change.
   */
  public int getNSteps() {
    return this.nSteps;
  }
  
  /**
   * Sets number of steps used for the state change.
   */
  public void setNSteps(int nSteps) {
    if (nSteps <= 0) {
      throw new IllegalArgumentException("non-positive number of steps");
    }
    this.nSteps = nSteps;
  }
  
  /**
   * Sets system to start values and prepares stepping.
   */
  public void restart() {
    if (system != null) {
      system.setState(pStart, vStart);
      nCur = 0;
    }
  }
  
  /**
   * Proceeds the associated thermodynamical system by one step.
   * At the first step it simply sends the current values.<br>
   * If the number of steps is one, the system proceeds to the final value
   * immediately.
   */
  public void nextStep() {
    if (nSteps == 1) {
      if (isIsochoric()) {
        system.changeState(n, ThermodynamicStateVariable.P, pEnd);
      } else {
        system.changeState(n, ThermodynamicStateVariable.V, vEnd);
      }
    } else {
      if (nCur != 0) {
        if (isIsochoric()) {
          double pNext = pStart + (pEnd - pStart)*nCur/nSteps;
          system.changeState(n, ThermodynamicStateVariable.P, pNext);
        } else {
          double vNext = vStart + (vEnd - vStart)*nCur/nSteps;
          system.changeState(n, ThermodynamicStateVariable.V, vNext);
        }
      }
      nCur++;
    }
  }
  
  /**
   * Returns true, if the state change is isochoric.
   */
  public boolean isIsochoric() {
    if (n == Double.POSITIVE_INFINITY) {
      return true;
    }
    
    double d = Math.pow(vStart, n);
    if ((vStart < 1 && d == 0.0) || (vStart > 1 &&  1.0/d == 0.0)) {
      return true;          // n == infinity numerically !
    }
    
    return false;
  }
  
  /**
   * Gets end point of the state change.
   * If it is isochoric the end point defines the pressure, else the volume.
   */
  public double getFinalPoint() {
    if (isIsochoric()) {
      return getFinalPressure();
    } else {
      return getFinalVolume();
    }
  }
  
  /**
   * Sets end point of the state change.
   * If it is isochoric the end point defines the pressure, else the volume.
   */
  public void setFinalPoint(double val) {
    if (isIsochoric()) {
      setFinalPressure(val);
    } else {
      setFinalVolume(val);
    }
  }
}
