package physbeans.phys;

import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * A lens using the paraxial approximation, but not necessarily thin. Uses
 * two principal planes to correct for effects of the lens width.
 */
public class ThickLens extends Lens implements Serializable {
  
  /** Creates a new instance of ThickLens */
  public ThickLens() {
  }
  
  /**
   * Extends a light ray through the optical element.
   */
  protected void extend(LightRay ray) {
    if (rayIsDeparting(ray)) {
      return;
    }
    
    DVector pLast = ray.getLastPoint();
    double al = ray.getAngle();
    
    double y1 = getHittingHeight(pLast, al);
    if (Math.abs(y1) > getHeight()) {
      return;         // miss
    }
    ray.addPoint(getFirstPrincipalPlane(), y1);
    ray.addPoint(getSecondPrincipalPlane(), y1);
    ray.setAngle(getApproximateRefractionAngle(pLast, y1));
  }
  
  /**
   * Returns the refraction angle of an incoming ray using the paraxial
   * approximation. The incoming ray is specified by the start point and
   * the hit height at the principal plane.
   * Returns NaN, if the start point is between the principal planes.
   */
  protected double getApproximateRefractionAngle(DVector pStart, double yHit) {
    DVector pImg = getImagePoint(pStart);
    double beta;
    if (Double.isInfinite(pImg.x())) {
      beta = Math.atan(pStart.y()/(pStart.x() - getFirstPrincipalPlane()));
    } else {
      beta = Math.atan((yHit - pImg.y())/(getSecondPrincipalPlane() - pImg.x()));
    }
    
    if (pStart.x() > pos) {
      beta -= Math.PI;
    }
    
    if ((pStart.x() > getFirstPrincipalPlane()) && 
        (pStart.x() < getSecondPrincipalPlane())) {
      beta = Double.NaN;
    }
    
    return beta;
  }
  
}
