package physbeans.phys;

import physbeans.math.*;
import physbeans.model.*;

/**
 * Base class of all medium configurations that can be set up in the ripple tank.
 * <P>The medium is given by a grid of wave velocity values (to be precise: by
 * the square of the velocity).
 */
public abstract class WaveMedium extends PhysModel implements GridWithBoundary {
  
  protected Grid2DScalarField field;
  protected double[][] grid;
  
  /** Creates a new instance of WaveWall */
  public WaveMedium() {
    field = null;
    setup(field);
  }
  
  /**
   * Sets up the wall.<br>
   * This has to be called each time the field dimensions change.
   */
  public void setup(Grid2DScalarField f) {
    if (f != null) {
      field = f;
      int nx = field.getGridSizeX();
      int ny = field.getGridSizeY();
      if ((grid == null) || (grid[0].length != nx) || (grid.length != ny)) {
        grid = new double[ny][nx];
      }
      setGrid();
    }
  }
  
  /**
   * Sets all grid cells to empty.
   */
  protected void initGrid() {
    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    
    for (int i = 0; i < nx; i++) {
      for (int j = 0; j < ny; j++) {
        grid[j][i] = 0.0;
      }
    }
  }
  
  /**
   * Sets the grid points that make up the wall.
   */
  abstract protected void setGrid();
  
  /**
   * Gets the grid of doubles values defining the wall.<br>
   * Uses doubles instead of booleans to make it compatible with
   * GridWithBoundary. This allows to use the generic GridView. Allowed values
   * are 1.0 (wall) and 0.0 (no wall).
   */
  public double[][] getGrid() {
    return grid;
  }
  
  // functions to implement the interface
  // grid dimensions are given by the underlying field
  
  public int getBoundaryWidthX() {
    if (field != null) {
      return field.getBoundaryWidthX();
    } else {
      return 0;
    }
  }
  
  public int getBoundaryWidthY() {
    if (field != null) {
      return field.getBoundaryWidthY();
    } else {
      return 0;
    }
  }
  
  public int getGridSizeX() {
    if (field != null) {
      return field.getGridSizeX();
    } else {
      return 0;
    }
  }
  
  public int getGridSizeY() {
    if (field != null) {
      return field.getGridSizeY();
    } else {
      return 0;
    }
  }
  
  /**
   * Gets vector (xMin, yMin).
   */
  public DVector getMinWorld() {
    return field.getMinWorld();
  }
  
  /**
   * Gets vector (xMax, yMax).
   */
  public DVector getMaxWorld() {
    return field.getMaxWorld();
  }
  
}
