package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;

import physbeans.inout.*;
import physbeans.phys.*;
import physbeans.math.*;
import physbeans.model.*;

import static java.lang.Math.*;
import static physbeans.math.Maths.*;

/**
 * Displays a Duffing oscillator as a vertical bar between two magnets.<br>
 * The external force is visualized as a horizontal movement of the magnets.
 * <p>
 * For the standard parameters a good LayeredScreen area is
 *   (-0.8, 0.1) .. (0.8, -1.1).
 */
public class DuffingView extends RestartableView {
  
  protected double l, w, mx, my, mw, sx, sf;
  protected Color colBar, colMag;
  
  /**
   * Creates a DuffingView with standard parameters.
   */
  public DuffingView() {
    l = 0.9;
    w = 1.5;
    mx = 0.5;
    my = -1.0;
    mw = 0.15;
    sx = 0.3;
    sf = 0.1;
    colBar = Color.BLACK;
    colMag = Color.RED;
  }
  
  /**
   * Redraws the image according to the input parameters.
   */
  public void paintLayer(Graphics2D g) {
    // draw the floor and the ceiling
    Line2D line = new Line2D.Double();
    line.setLine(-w/2, 0.0, w/2, 0.0);
    g.draw(line);
    line.setLine(-w/2, my, w/2, my);
    g.draw(line);
    
    // draw the bar
    double q0 = sx*model.get(0);
    CubicCurve2D bar = new CubicCurve2D.Double(0.0, 0.0, 0.0, -l/3.0,
        q0/2.0, -2.0*l/3.0, q0, -l);
    // computation of the control points is non-trivial
    // cf. technical notes
    g.setColor(colBar);
    g.draw(bar);
    
    // draw the magnets
    double dx = sf*model.get(1);
    Rectangle2D mag1 = new Rectangle2D.Double(mx - 0.5*mw - dx, my, mw, mw);
    Rectangle2D mag2 = new Rectangle2D.Double(-mx - 0.5*mw - dx, my, mw, mw);
    g.setColor(colMag);
    g.fill(mag1);
    g.fill(mag2);
  }
  
  /**
   * Gets the length of the bar.
   */
  public double getBarLength() {
    return this.l;
  }
  
  /**
   * Sets the length of the bar.
   */
  public void setBarLength(double l) {
    this.l = l;
    update();
  }
  
  /**
   * Gets the total width of the horizontal image.
   */
  public double getTotalWidth() {
    return this.w;
  }
  
  /**
   * Sets the total width of the horizontal image.
   */
  public void setTotalWidth(double w) {
    this.w = w;
    update();
  }
  
  /**
   * Gets the x position (x > 0) of the right magnet.
   * The other magnet sits at -x.
   */
  public double getMagnetPositionX() {
    return this.mx;
  }
  
  /**
   * Sets the x position (x > 0) of the right magnet.
   * The other magnet sits at -x.
   */
  public void setMagnetPositionX(double mx) {
    this.mx = mx;
    update();
  }
  
  /**
   * Gets the y position (usually negative) of the magnets.
   */
  public double getMagnetPositionY() {
    return this.my;
  }
  
  /**
   * Sets the y position (usually negative) of the magnets.
   */
  public void setMagnetPositionY(double my) {
    this.my = my;
    update();
  }
  
  /**
   * Gets width of the quadratic magnets.
   */
  public double getMagnetWidth() {
    return this.mw;
  }
  
  /**
   * Gets width of the quadratic magnets.
   */
  public void setMagnetWidth(double mw) {
    this.mw = mw;
    update();
  }
  
  /**
   * Gets the scale factor for the displacement input.
   */
  public double getXScale() {
    return this.sx;
  }
  
  /**
   * Sets the scale factor for the displacement input.
   */
  public void setXScale(double sx) {
    this.sx = sx;
    update();
  }
  
  /**
   * Gets the scale factor for the force input.
   */
  public double getFScale() {
    return this.sf;
  }
  
  /**
   * Sets the scale factor for the force input.
   */
  public void setFScale(double sf) {
    this.sf = sf;
    update();
  }
  
  /**
   * Gets the color of the bar.
   */
  public Color getBarColor() {
    return this.colBar;
  }
  
  /**
   * Sets the color of the bar.
   */
  public void setBarColor(Color colBar) {
    this.colBar = colBar;
    update();
  }
  
  /**
   * Gets the color of the magnets.
   */
  public Color getMagnetColor() {
    return this.colMag;
  }
  
  /**
   * Sets the color of the magnets.
   */
  public void setMagnetColor(Color colMag) {
    this.colMag = colMag;
    update();
  }
  
  /**
   * Sets the parameter defining the displacement of the oscillator.
   * Conveniance function to simplify routing.
   */
  public void setDisplacement(double x) {
    model.set(0, x);
    update();
  }
  
  /**
   * Sets the parameter defining the external force on the oscillator.
   * Conveniance function to simplify routing.
   */
  public void setForce(double f) {
    model.set(1, f);
    update();
  }}
