package physbeans.views;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;


/**
 * View that draws a color image representing a scalar field.<br>
 * It uses a color table with 256 entries to map scalar values to colors.
 */
public class ImageView extends View implements Serializable {
  protected ScalarField field;
  protected double minVal, maxVal;
  protected boolean infiniteScale;
  protected double scale;
  protected ColorTable colors;
  
  protected BufferedImage image;
  protected DataBufferByte buffer;
  
  /** Creates a new instance of ImageView */
  public ImageView() {
    super();
    
    // create a standard colortable
    colors = new ColorTable();
    
    minVal = -1.0;
    maxVal = 1.0;
    infiniteScale = false;
    scale = 1.0;
  }
  
  /**
   * calculates the image, using the known size
   */
  public void doLayout() {
    // get dimension of drawing area from panel
    Point dim = trafo.getMaxScreen();
    int size = dim.x*dim.y;
    if (size <= 0) {
      return;
    }
    
    // create a new image with a new buffer size
    createImage();
    
    // calculate the values for the buffer from the field
    calcImageBuffer();
  }
  
  public void paintLayer(Graphics2D g2) {
    g2.drawImage(image, trafo.getInverseAffineTransform(), myParent);
  }
  
  /**
   * Gets scalar value that is mapped to highest color (255).
   */
  public double getMaxVal() {
    return maxVal;
  }
  
  /**
   * Sets scalar value that is mapped to highest color (255).<p>
   * Settings maxVal to Infinity leads to scaling with arctan
   */
  public void setMaxVal(double max) {
    maxVal = max;
    update();
  }
  
  /**
   * Gets scalar value that is mapped to lowest color (0).
   */
  public double getMinVal() {
    return minVal;
  }
  
  /**
   * Sets scalar value that is mapped to lowest color (0).
   */
  public void setMinVal(double min) {
    minVal = min;
    update();
  }
  
  /**
   * Gets scalar field to be displayed.
   */
  public ScalarField getModel() {
    return this.field;
  }
  
  /**
   * Sets scalar field to be displayed.
   */
  public void setModel(ScalarField model) {
    this.field = model;
    update();
  }
  
  /**
   * calculates the image buffer from the field data
   * faster version (cf. calcImageBufferSimple): <br>
   *    gets increment in world coordinates
   *    steps through pixels
   *    use increment to get world coordinates
   * spares additional trafo call in each step
   */
  protected void calcImageBuffer() {
    if (buffer == null || field == null) {
      return;
    }
    
    double val; // current field value
    long valIndex; // index value corresponding to val
    
    // get world parameter before the loop
    // use them inside loop to spare trafo calls
    Point dim = trafo.getMaxScreen();
    DVector minWorld = trafo.getMinWorld();
    DVector maxWorld = trafo.getMaxWorld();
    DVector oneStep = trafo.transform(new Point(1, 1));
    double minX = minWorld.get(0);
    double maxY = maxWorld.get(1);
    double decX = oneStep.get(0) - minX;
    double decY = oneStep.get(1) - maxY;    // decY < 0 !
    
    // loop through image and world at once
    
    // scale down infinite range
    if (infiniteScale) {
      double x = minX;
      for (int s = 0; s < dim.x; s++, x += decX) {
        double y = maxY;
        for (int t = 0; t < dim.y; t++, y += decY) {
          val = field.getScalarValue(x, y);
          // compress (-infinity .. infinity) to (-1 .. 1)
          val = 2 / Math.PI * Math.atan(val/scale);
          buffer.setElem(s + dim.x * t, (byte) Math.round(255.0 * (val + 1.0) / 2.0));
        }
      }
    } else {
      double x = minX;
      for (int s = 0; s < dim.x; s++, x += decX) {
        double y = maxY;
        for (int t = 0; t < dim.y; t++, y += decY) {
          val = field.getScalarValue(x, y);
          valIndex = Math.round(255.0 * (val - minVal) / (maxVal - minVal));
          buffer.setElem(s + dim.x * t, (byte) Math.max(0, Math.min(255, valIndex)));
        }
      }
    }
  }
  
  /**
   * Creates the image and get the buffer for the pixel values.
   */
  protected void createImage() {
    // create new image with buffer
    Point dim = trafo.getMaxScreen();
    image = new BufferedImage(dim.x, dim.y, BufferedImage.TYPE_BYTE_INDEXED,
        colors.getTable());
    WritableRaster raster = image.getRaster();
    buffer = (DataBufferByte) raster.getDataBuffer();
  }
  
  /**
   * Recalculates and redraws the image using the current buffer.<br>
   */
  public void update() {
    Point dim = trafo.getMaxScreen();
    if (dim.x * dim.y <= 0) {
      return;
    }
    calcImageBuffer();
    super.update();
  }
  
  /**
   * Gets value of infiniteScale flag.<br>
   * If true, values are scaled using atan and a scale factor, 
   * otherwise linearly between minVal and maxVal.
   */
  public boolean isInfiniteScale() {
    return this.infiniteScale;
  }
  
  /**
   * Sets value of infiniteScale flag.<br>
   * If true, values are scaled using atan and a scale factor, 
   * otherwise linearly between minVal and maxVal.
   */
  public void setInfiniteScale(boolean infiniteScale) {
    this.infiniteScale = infiniteScale;
    update();
  }
  
  /**
   * Gets scale factor used with infinite scaling.
   */
  public double getScale() {
    return this.scale;
  }
  
  /**
   * Sets scale factor used with infinite scaling.
   */
  public void setScale(double scale) {
    this.scale = scale;
    update();
  }
  
  /**
   * Gets color table.
   */
  public ColorTable getColors() {
    return this.colors;
  }
  
  /**
   * Sets color table.
   */
  public void setColors(ColorTable colors) {
    this.colors = colors;
  }
  
}
