package physbeans.views;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * Displays a rectangular array of colored disks, representing a state vector.<br>
 * The states are displayed using different colors.
 */
public class ParticleStateView extends View implements Serializable {
  
  protected StateVector model;
  
  // colors of the particles
  // if there are more types than colors, this is used cyclically
  protected Color[] particleColor;
  protected int minCols;
  protected Ellipse2D[] circle;
  
  /**
   * Creates a default ParticleStateView.
   */
  public ParticleStateView() {
    super();
    minCols = 10;
    
    particleColor = new Color[] {new Color(0, 200, 0), new Color(200, 0, 200),
    new Color(0, 200, 200), Color.pink, Color.orange, Color.red,
    Color.blue, Color.yellow};
  }
  
  /**
   * Gets underlying model defining the points to paint.
   */
  public StateVector getModel() {
    return model;
  }
  
  /**
   * sets underlying model defining the points to paint
   */
  public void setModel(StateVector model) {
    this.model = model;
    update();
  }
  
  /**
   * Do all initialisations that can only be done after the layout (ie the size)
   * is fixed.
   */
  public void doLayout() {
    createGrid();
  }
  
  /**
   * Gets the minimal number of columns.
   */
  public int getMinimalNumberOfColumns() {
    return minCols;
  }
  
  /**
   * Sets the minimal number of columns.
   */
  public void setMinimalNumberOfColumns(int n) {
    minCols = n;
    createGrid();
    update();
  }
  
  /**
   * Gets color of a particle type.
   */
  public Color getParticleColor(int index) {
    return this.particleColor[index];
  }
  
  /**
   * Gets colors of all particle types.
   */
  public Color[] getParticleColor() {
    return this.particleColor;
  }
  
  /**
   * Sets color of a particle type.
   */
  public void setParticleColor(int index, Color particleColor) {
    this.particleColor[index] = particleColor;
    update();
  }
  
  /**
   * Sets colors of all particle types.
   */
  public void setParticleColor(Color[] particleColor) {
    this.particleColor = particleColor;
    update();
  }
  
  /**
   * Creates rectangular grid of particle positions.
   */
  protected void createGrid() {
    if (model == null) {
      return;
    }
    
    // setup circles representing the particles
    int nPart = model.getNumberOfElements();
    circle = new Ellipse2D[nPart];
    for (int i = 0; i < nPart; i++) {
      circle[i] = new Ellipse2D.Double();
    }
    
    DVector v = computeDimensions();
    double width = v.get(0);
    double height = v.get(1);
    int nCols = (int) v.get(2);
    
    // compute new particle positions
    for (int i = 0; i < nPart; i++) {
      double x = width*(i % nCols) + trafo.getMinX();
      double y = height*(i / nCols) + trafo.getMinY();
      circle[i].setFrame(x, y, width, height);
    }
  }
  
  /**
   * Paints the circles at precomputed positions, using the colors for the
   * current states.
   */
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;
    }
    
    if (circle == null || circle.length != model.getNumberOfElements()) {
      createGrid();
    }
    
    for (int i = 0; i < model.getNumberOfElements(); i++) {
      g2.setColor(particleColor[model.getState(i) % particleColor.length]);
      g2.fill(circle[i]);
    }
  }
  
  /**
   * Computes parameters for optimal fitting of the circles.
   * Returns width, height, nCols.
   */
  protected DVector computeDimensions() {
    int nPart = model.getNumberOfElements();
    int xMax = trafo.getMaxS();
    int yMax = trafo.getMaxT();
    int nCols = (int) Math.ceil(Math.sqrt(nPart * xMax / (double) yMax)) + 1;
    nCols = Math.max(nCols, minCols);
    
    double r = xMax / (double)nCols;
    double width = r*trafo.getScalingX();
    double height = r*trafo.getScalingY();
    return new DVector(width, height, nCols);
  }
}
