package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;

import physbeans.inout.ScreenWorldTrafo;
import physbeans.phys.*;
import physbeans.math.*;
import physbeans.model.*;

/**
 * View displaying a PlaneLightSource and its light rays.
 * Displays the source as a small headlight.
 */
public class PlaneLightSourceView extends View {
  
  protected PlaneLightSource model;
  protected LightRayBundleView raysView;
  protected Color sourceColor;
  protected double scale;
  
  /** Creates a new instance of PointLightSourceView */
  public PlaneLightSourceView() {
    raysView = new LightRayBundleView();
    sourceColor = new Color(255, 204, 0);
    scale = 1.1;
  }
  
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;  // do nothing
    }
    
    double x = model.getPosition();
    double y = model.getHeight();
    double rx = scale*model.getWidth()/2;
    double ry = rx * trafo.getRelativeScaling();
    double alpha = model.getAngle();
    
    // create a half circle for the headlight
    Shape arc = new Arc2D.Double(x - rx, y - ry, 2*rx, 2*ry,
                                 90.0, 180.0, Arc2D.CHORD);
    AffineTransform rotator = AffineTransform.getRotateInstance(alpha, x, y);
    arc = rotator.createTransformedShape(arc);
        
    raysView.setModel(model.getRays());
    raysView.paintLayer(g2);
    
    g2.setColor(sourceColor);
    g2.fill(arc);
    
    g2.setColor(Color.black);
    g2.draw(arc);
  }
  
  /**
   * Gets color of the lines.
   */
  public Color getRayColor() {
    return raysView.getLineColor();
  }
  
  /**
   * Sets color of the lines.
   */
  public void setRayColor(Color lineColor) {
    raysView.setLineColor(lineColor);
    update();
  }
  
  /**
   * Sets trafo between screen and world coordinates.
   */
  public void setTrafo(ScreenWorldTrafo trafo) {
    super.setTrafo(trafo);
    raysView.setTrafo(trafo);
  }
  
  /**
   * Gets the color of the figure representing the source.
   */
  public Color getSourceColor() {
    return this.sourceColor;
  }
  
  /**
   * Sets the color of the figure representing the source.
   */
  public void setSourceColor(Color sourceColor) {
    this.sourceColor = sourceColor;
  }
  
  
  /**
   * Gets underlying model defining the light source to paint.
   */
  public PlaneLightSource getModel() {
    return model;
  }
  
  /**
   * Sets underlying model defining the light source to paint.
   */
  public void setModel(PlaneLightSource model) {
    this.model = model;
    update();
  }
}
